#include <stdio.h>
#include <qimage.h>
#include <qmime.h>
#include <qfile.h>
#ifdef MAC
 #include <qsound.h>
 #define QBOOK_TEMP_WAV "/home/tmp/ztenv.wav"
#endif
#include <qpainter.h>
#include <qprogressbar.h>
#include <stdlib.h>
#include <eb/binary.h>
#include "qbookentry.h"
#include "diskstat.h"
#include "utils.h"
#include "ztenconf.h"
#include "qbookbrowser.h"
#include "ztensound.h"
#include "ztenimage.h"
//#include "ztentext.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <cstring>

#include <qbitmap.h>

extern "C"
{
  EB_Error_Code eb_find_file_name3(const char *path_name,
     const char *sub_directory_name, const char *sub2_directory_name,
     const char *target_file_name, char *found_file_name);
  EB_Error_Code eb_find_file_name2(const char *path_name,
     const char *sub_directory_name, 
     const char *target_file_name, char *found_file_name);

  // #include <libkakasi.h>
  extern void* z_kakasi_dlhandle;
  extern int (*z_kakasi_getopt_argv)(int argc, char **argv);
  extern char* (*z_kakasi_do)(char *str);
  extern int (*z_kakasi_close_kanwadict)(void);
  extern int (*z_kakasi_free)(char *p);

  int zten_stop_code0, zten_stop_code1;
}

#ifdef QTOPIA
  #include <qpe/qpeapplication.h>
  extern QPEApplication *app;
#else
  #include <qapplication.h>
  extern QApplication *app;
#endif

#if 0
#include <dlfcn.h>
extern "C" {
  static int (*zz_decode_file)(const char *infile, const char *outfile);
}
#endif

#define COLOR_DEPTH 32

static const char* INSERT_MARK = "\x01";
static const char* GB2312_MARK = "__GB2312__"; // must be in Ascii Text String
unsigned int last_wide_font = 0;
unsigned int last_narrow_font = 0;
bool UseSubSupFont;

int refCount = 0;

QBookBrowser* qbb;



inline unsigned int getDWord(const unsigned char *ubuf) {
  return 256*256*256*ubuf[0]+256*256*ubuf[1]+256*ubuf[2]+ubuf[3];
}

inline unsigned short getWord(const unsigned char *ubuf) {
  return 256*ubuf[0]+ubuf[1];
}

inline unsigned int getDWord(const char *buf) {
  return getDWord((const unsigned char *) buf);
}

inline unsigned short getWord(const char *buf) {
  return getWord((const unsigned char *) buf);
}

inline int getBCD2(const unsigned char *buf) {
  return 10*((0xf0 & *buf)>>4) + (0xf & *buf);
}

inline int getBCD4(const unsigned char *buf) {
  return 100*getBCD2(buf)+getBCD2(buf+1);
}

inline int getBCD8(const unsigned char *buf) {
  return 10000*getBCD4(buf)+getBCD4(buf+2);
}

inline int getBCD2(const char *buf) {
  return getBCD2((const unsigned char *)buf);
}
inline int getBCD4(const char *buf) {
  return getBCD4((const unsigned char *)buf);
}
inline int getBCD8(const char *buf) {
  return getBCD8((const unsigned char *)buf);
}

int
getBinary(EB_Book *book, char** buf)
{
  int length_read, size;
  size = 0;
  *buf = (char*)malloc(MAX_LENGTH);

  int len=book->binary_context.size;
  if (len<0 || len>MAX_BINARY) len=MAX_BINARY;

  /**/
  QProgressBar *p=new QProgressBar(len,qbb);
  p->move(10,240);
  p->resize(200,40);
  p->show();
  p->reset();
  /**/

  do
    {
      if (eb_read_binary(book, MAX_LENGTH, *buf + size, &length_read)
	  != EB_SUCCESS)
	break;
      size += length_read;
      *buf = (char*)realloc(*buf, size + MAX_LENGTH);
      if (size >= MAX_BINARY)
	break;
      /**/      p->setProgress(size);
    }
  while (length_read > 0);

  /**/
  p->setProgress(len);
  delete p;//p->close(TRUE);
  /**/
  return size;
}

int
getJpegBinary(EB_Book *book, char** buf)
{
  int length_read, size, length_next = 0;
  size = 0;
  *buf = (char*)malloc(2);
  eb_read_binary(book, 2, *buf, &length_read);
  size += 2;


  /**/
  QProgressBar *p=new QProgressBar(MAX_BINARY,qbb);
  p->move(10,240);
  p->resize(200,40);
  p->show();
  p->reset();
  /**/

  while(1) {
    *buf = (char*)realloc(*buf, size + 2);
    eb_read_binary(book, 2, *buf + size, &length_read); 
    size += 2;
    if ((*buf)[size-1] == (char)0xd9)
      break;

    *buf = (char*)realloc(*buf, size + 2);
    eb_read_binary(book, 2, *buf + size, &length_read); 
    size += 2;

    length_next = (*buf)[size-2]*0x100 + (*buf)[size-1] - 2;
    if (length_next < 0) break;
    *buf = (char*)realloc(*buf, size + length_next);
    eb_read_binary(book, length_next, *buf + size, &length_read); 
    size += length_next;
    /**/    p->setProgress(size);
  }
  /**/
  p->setProgress(MAX_BINARY);
  delete p;
  /**/

  return size;
}

// Hooks for EB.

EB_Error_Code
text_hook_null (EB_Book*, EB_Appendix*, void *, EB_Hook_Code, 
		int, const unsigned int*)
{
  return 0;
}

static bool isHookKeyword = false;
static bool isInReference = false;

EB_Error_Code
hook_initialize (EB_Book*, EB_Appendix*, void *, EB_Hook_Code, 
		 int, const unsigned int*)
{
  isHookKeyword = false;
  isInReference = false;
  return 0;
}

EB_Error_Code
text_hook_space (EB_Book* book, EB_Appendix*, void *, EB_Hook_Code, 
		 int, const unsigned int*)
{
  if(config->SpaceSummarized) {
    eb_write_text_string(book, " ");
  }
  else {
    eb_write_text_string(book, "<span> </span>");
  }
  return 0;
}

EB_Error_Code
text_hook_wide_space (EB_Book* book, EB_Appendix*, void *, EB_Hook_Code, 
		      int, const unsigned int*)
{
  if(config->SpaceSummarized) {
    eb_write_text_string(book, "  ");
  }
  else {
    eb_write_text_string(book, "<span> </span><span> </span>");
  }
  return 0;
}


static EB_Position last_newline;

EB_Error_Code
text_hook_newline (EB_Book* book, EB_Appendix*,
		   void*, EB_Hook_Code, 
		   int, const unsigned int*)
{
  eb_write_text_string(book, "<p>");
  if (eb_tell_text(book, &last_newline)!=EB_SUCCESS) {
    last_newline.page=last_newline.offset=0;
  } else {
    last_newline.offset+=2;
    if (last_newline.offset>=2048) {
      last_newline.page++;
      last_newline.offset-=2048;
    }
  }
  return 0;
}

EB_Error_Code
text_hook_narrow_font (EB_Book *book, EB_Appendix*,
		       void *obj, EB_Hook_Code, 
		       int, const unsigned int* argv)
{
  char c [100];
  char bitmap[EB_SIZE_NARROW_FONT_48];

  QBookEntry *be = (QBookEntry *) obj;

  EB_Error_Code rc = eb_narrow_font_character_bitmap(book, argv[0], bitmap);
  if (rc != EB_SUCCESS)
    {
      eb_write_text_string (book, "&nbsp;");
      return 0;
    }

  if (last_narrow_font != argv[0])
    {
      // FIXME: there should be a better way -- from qtjiten
      QImage qim(8, 16, 1, 2, QImage::systemBitOrder());
      for(int y=0; y<16; y++) 
	for(int x=0; x<8; x++)
	  qim.setPixel(x, y, (bitmap[y] & (1 << (7-x))) ? 1:0);
      sprintf(c, "n-font_n%x", argv[0]);
#ifdef QTOPIA
      qim.setColor(1, Qt::color0.rgb());
      qim.setColor(0, Qt::color1.rgb());
      QMimeSourceFactory::defaultFactory()->setImage(c, qim.convertDepth(32));
#else
      qim.setColor(0, Qt::color0.rgb());
      qim.setColor(1, Qt::color1.rgb());
      QMimeSourceFactory::defaultFactory()->setImage(c, qim);
#endif

      QImage qim2=qim.copy();
      sprintf(c, "h-font_n%x", argv[0]);
#ifdef QTOPIA
      qim2.setColor(1, Qt::color0.rgb());
      qim2.setColor(0,QBookEntry::back_color);
      QMimeSourceFactory::defaultFactory()->setImage(c, qim2.convertDepth(32));
#else
      qim2.setColor(0,QBookEntry::back_color);
      qim2.setColor(1,QBookEntry::fore_color);
      QMimeSourceFactory::defaultFactory()->setImage(c, qim2);
#endif
    }
  last_narrow_font = argv[0];

  sprintf(c, "<img src=\"n-font_n%x\" alt=\"GAIJI(%x)\" width=%d height=%d>",
	  argv[0], argv[0], be->getFontSize () / 2, be->getFontSize ());
  eb_write_text_string(book, c);

  return 0;
}

EB_Error_Code
text_hook_wide_font (EB_Book* book, EB_Appendix*, void *obj, 
		     EB_Hook_Code,
		     int, const unsigned* argv)
{
  char c [100];
  char bitmap[EB_SIZE_WIDE_FONT_48];
  QBookEntry *be = (QBookEntry *) obj;

  EB_Error_Code rc = eb_wide_font_character_bitmap(book, argv[0], bitmap);
  if (rc != EB_SUCCESS)
    {
      eb_write_text_string (book, "&nbsp;&nbsp;");
      return 0;
    }

  if (last_wide_font != argv[0])
    {
      // FIXME: there should be a better way -- from qtjiten
      QImage qim(16, 16, 1, 2, QImage::systemBitOrder());
      for(int y=0; y<16; y++) {
	for(int x=0; x<16; x++) {
	  qim.setPixel(x, y, (bitmap[y*2+((x<8)?0:1)] & (1 << (7-(x%8)))) ? 1:0);
	}
      }
      sprintf(c, "n-font_w%x", argv[0]);
#ifdef QTOPIA
      qim.setColor(1, Qt::color0.rgb());
      qim.setColor(0, Qt::color1.rgb());
      QMimeSourceFactory::defaultFactory()->setImage(c, qim.convertDepth(32));
#else
      qim.setColor(0, Qt::color0.rgb());
      qim.setColor(1, Qt::color1.rgb());
      QMimeSourceFactory::defaultFactory()->setImage(c, qim);
#endif

      QImage qim2=qim.copy();
      sprintf(c, "h-font_w%x", argv[0]);
#ifdef QTOPIA
      qim2.setColor(1, Qt::color0.rgb());
      qim2.setColor(0,QBookEntry::back_color);
      QMimeSourceFactory::defaultFactory()->setImage(c, qim2.convertDepth(32));
#else
      qim2.setColor(0,QBookEntry::back_color);
      qim2.setColor(1,QBookEntry::fore_color);
      QMimeSourceFactory::defaultFactory()->setImage(c, qim2);
#endif
    }
  last_wide_font = argv[0];

  sprintf(c, "<img src=\"n-font_w%x\" alt=\"GAIJI(%x)\" width=%d height=%d>",
	  argv[0], argv[0], be->getFontSize (), be->getFontSize ());
  eb_write_text_string(book, c);

  return 0;
}

EB_Error_Code
text_hook_narrow_jisx0208 (EB_Book* book, EB_Appendix *appendix, 
				       void *container, 
				       EB_Hook_Code code, 
				       int argc, const unsigned int* argv)
{
  if (argv[0] == 0xa1e3)
    eb_write_text_string(book, "&lt;");
  else if (argv[0] == 0xa1e4)
    eb_write_text_string(book, "&gt;");
  else if (argv[0] == 0xa1f5)
    eb_write_text_string(book, "&amp;");
  else if (argv[0] == 0xa1a1 && !config->SpaceSummarized)
    eb_write_text_string(book, "<span> </span>");
  else 
    eb_hook_euc_to_ascii(book, appendix, container, code, argc, argv);
  return 0;
}

EB_Error_Code
text_hook_gb2312 (EB_Book* book, EB_Appendix * /*appendix*/,
		  void * /* container */, EB_Hook_Code /* code */,
		  int /* argc */, const unsigned int* argv) {
  char buf[10];
  sprintf(buf,"%s%02x%02x",GB2312_MARK,
	  0x80 | (0xff & (argv[0] >> 8)),
	  0x80 | (0xff & argv[0]));
  eb_write_text_string (book, buf);
  return 0;
}
		  

static bool in_subsup=false;

EB_Error_Code
text_hook_begin_superscript (EB_Book* book, EB_Appendix*, 
			     void * /* obj */ , 
			     EB_Hook_Code,
			     int, const unsigned* /* argv */)
{
  if (UseSubSupFont) {
    eb_write_text_string(book, "<font face=\"ebsup\">");
  } else {
    eb_write_text_string(book, "^{");
  }
  in_subsup=true;
  return 0;
}

EB_Error_Code
text_hook_begin_subscript (EB_Book* book, EB_Appendix*, void * /* obj */, 
			   EB_Hook_Code,
			   int, const unsigned* /* argv */)
{
  if (UseSubSupFont) {
    eb_write_text_string(book, "<font face=\"ebsub\">");
  } else {
    eb_write_text_string(book, "<font size=-1>");
  }
  in_subsup=true;
  return 0;
}

EB_Error_Code
text_hook_end_subscript (EB_Book* book, EB_Appendix*, void * /* obj */, 
			 EB_Hook_Code,
			 int, const unsigned* /* argv */)
{
  if (in_subsup) {
    eb_write_text_string(book, "</font>");
  }
  in_subsup=false;
  return 0;
}

EB_Error_Code
text_hook_end_superscript (EB_Book* book, EB_Appendix*, void * /* obj */, 
			   EB_Hook_Code,
			   int, const unsigned* /* argv */)
{
  if (in_subsup) {
    if (UseSubSupFont) {
      eb_write_text_string(book, "</font>");
    } else {
      eb_write_text_string(book, "}");
    }
  }
  in_subsup=false;
  return 0;
}

#define DECO_MAX 10
static int decoration[DECO_MAX];
static int n_decoration=0;

EB_Error_Code
text_hook_begin_decoration(EB_Book* book, EB_Appendix*, void * /* obj */, 
		      EB_Hook_Code, 
		      int, const unsigned int* argv)
{
  //  QBookEntry *be = (QBookEntry *) obj;  

  if (0<=n_decoration && n_decoration<DECO_MAX) {
    decoration[n_decoration]=argv[1];
    if (argv[1]==0x0001) {
      eb_write_text_string(book, "<i>");
    } else {
      eb_write_text_string(book, "<b>");
    }
  }
  n_decoration++;
  return EB_SUCCESS;
}

EB_Error_Code
text_hook_end_decoration(EB_Book* book, EB_Appendix*, void*, 
	      EB_Hook_Code, 
	      int, const unsigned int*) {
  n_decoration--;
  if (n_decoration<DECO_MAX && n_decoration>=0) {
    if (decoration[n_decoration]==0x0001) {
      eb_write_text_string(book, "</i>");
    } else {
      eb_write_text_string(book, "</b>");
    }
  }
  return EB_SUCCESS;
}

EB_Error_Code
text_hook_begin_emphasis(EB_Book* book, EB_Appendix*, void * /* obj */, 
			 EB_Hook_Code, 
			 int, const unsigned int* /* argv */)
{
  eb_write_text_string(book, "<em>");
  return EB_SUCCESS;
}

EB_Error_Code
text_hook_end_emphasis(EB_Book* book, EB_Appendix*, void * /* obj */, 
		       EB_Hook_Code, 
		       int, const unsigned int* /* argv */)
{
  eb_write_text_string(book, "</em>");
  return EB_SUCCESS;
}

static bool keyword_in=false;

EB_Error_Code
text_hook_begin_keyword(EB_Book* book, EB_Appendix*, void * /* obj */, 
			EB_Hook_Code, 
			int, const unsigned int* /* argv */)
{
  isHookKeyword = true;
  keyword_in=true;
  eb_write_text_string(book, "<strong>");
  return EB_SUCCESS;
}

EB_Error_Code
text_hook_end_keyword(EB_Book* book, EB_Appendix*, void * /* obj */, 
		      EB_Hook_Code, 
		      int, const unsigned int* /* argv */)
{
  if (keyword_in) 
    eb_write_text_string(book, "</strong>");
  keyword_in=false;
  return EB_SUCCESS;
}

EB_Error_Code
text_hook_graphic_reference (EB_Book* book, EB_Appendix*, void* obj, 
			   EB_Hook_Code, 
			   int, const unsigned int* argv)
{
  QBookEntry *be = (QBookEntry *) obj;  
  
  eb_write_text_string (book, INSERT_MARK);
  eb_write_text_string (book, "[Image]</a>");

  int index = be->mmarray.size();

  be->mmarray.resize(index+1);
  be->mmarray[index].type=MM_IMAGE_GROUP;
  be->mmarray[index].subType=0;
  be->mmarray[index].begin.page = argv[1];
  be->mmarray[index].begin.offset = argv[2];

  char c[32];
  sprintf (c, "%d_%d", LINK_IMAGE_GROUP, index);
  be->addPositionString ( new QString (c) );

  return EB_SUCCESS;
}

EB_Error_Code
text_hook_begin_image_group (EB_Book* book, EB_Appendix*, void* obj, 
		       EB_Hook_Code, 
		       int, const unsigned int* argv)
{
  isInReference = true;
  eb_write_text_string (book, " ");
  eb_write_text_string (book, INSERT_MARK);
  eb_write_text_string (book, "[Image]");

  /*
  QBookEntry *be = (QBookEntry *) obj;  
  int index = be->mmarray.size();

  be->mmarray.resize(index+1);
  be->mmarray[index].type=MM_IMAGE_GROUP;
  be->mmarray[index].subType=0;
  be->mmarray[index].begin.page = argv[1];
  be->mmarray[index].begin.offset = argv[2];

  char c[32];
  sprintf (c, "%d_%d", LINK_IMAGE_GROUP, index);
  be->addPositionString ( new QString (c) );
  */

  char c[32];
  QBookEntry *be = (QBookEntry *) obj;
  sprintf (c, "%d_%d", argv[1], argv[2]);
  be->addPositionString ( new QString (c) );

  return 0;
}

EB_Error_Code
text_hook_end_image_group (EB_Book* book, EB_Appendix*, void *, 
			   EB_Hook_Code, 
			   int, const unsigned int*)
{
  eb_write_text_string(book, "</a>");
  isInReference = false;
  return 0;
}

EB_Error_Code
text_hook_begin_reference (EB_Book* book, EB_Appendix*, void*, 
			   EB_Hook_Code, 
			   int, const unsigned int*)
{
  isInReference = true;
  eb_write_text_string (book, "&nbsp;");
  eb_write_text_string (book, INSERT_MARK);
  return 0;
}

EB_Error_Code
text_hook_end_reference (EB_Book* book, EB_Appendix*, void *obj, 
			 EB_Hook_Code, 
			 int, const unsigned int* argv)
{
  char c[32];
  QBookEntry *be = (QBookEntry *) obj;
  sprintf (c, "%d_%d", argv[1], argv[2]);
  be->addPositionString ( new QString (c) );
  eb_write_text_string(book, "</a>");
  isInReference = false;
  return 0;
}

// Image processing from qtjiten.
void
process_color (QBookEntry *be, EB_Book *book, EB_Position *p, bool inLine)
{
  if (eb_set_binary_color_graphic(book, p) != EB_SUCCESS)
    {
      qDebug("eb_set_binary_color_graphic failed");
      return;
    }

  int index = be->mmarray.size();
  be->mmarray.resize(index+1);
  be->mmarray[index].type = MM_IMAGE;
  be->mmarray[index].subType = be->img_type;
  be->mmarray[index].begin = *p;
  be->mmarray[index].width = be->img_width;
  be->mmarray[index].height = be->img_height;

  char *buf;
  int size;
  EB_Disc_Code disc_code;

  if (be->img_type == IMG_JPEG &&
      (eb_disc_type(book, &disc_code) != EB_SUCCESS ||
       disc_code != EB_DISC_EPWING))
    size = getJpegBinary(book, &buf);
  else 
    size = getBinary(book, &buf);

  QImage qim;
  qim.loadFromData((unsigned char*)buf, size);
  free(buf);

  int scale=config->ImageScale;
  if (inLine) scale = 100;
  if (config->ImageSizeLimitMode) {
    int sc0=100,sc1=100;
    if (qim.width()>config->MaxImageWidth) {
      sc0 = config->MaxImageWidth*100/qim.width();
    }
    if (qim.height()>config->MaxImageHeight) {
      sc1 = config->MaxImageHeight*100/qim.height();
    }
    if (sc0<scale) scale=sc0;
    if (sc1<scale) scale=sc1;
  }
  int w, h;
  w = (qim.width() * scale) / 100;
  h = (qim.height() * scale) / 100;

  char c[100]; // I guess this is enough
  sprintf(c, "image%d", be->img_no);
  if (scale==100) {
    QMimeSourceFactory::defaultFactory()->setImage(c, qim.convertDepth (COLOR_DEPTH));
  } else {
    QMimeSourceFactory::defaultFactory()->
      setImage(c, qim.smoothScale(w,h).convertDepth (COLOR_DEPTH));
  }

  if (scale<100) {
    sprintf (c, "%d_%d", LINK_IMAGE, be->mmarray.size()-1);
    be->addPositionString ( new QString (c) );

//    eb_write_text_string (book, "<p>");
    eb_write_text_string (book, INSERT_MARK);
    eb_write_text_string (book, "[Image]");
//    sprintf(c, "<img source=\"image%d\" width=%d height=%d></a></p>", 
//	    be->img_no, w, h);
    sprintf(c, "<img source=\"image%d\" width=%d height=%d></a>",
	    be->img_no, w, h);
    eb_write_text_string(book, c);
  } else {
//    sprintf(c, "<p><img source=\"image%d\" width=%d height=%d></p>", 
//	    be->img_no, w, h);
    sprintf(c, "<img source=\"image%d\" width=%d height=%d>", 
	    be->img_no, w, h);
    eb_write_text_string(book, c);
  }
  be->img_no++;
}

EB_Error_Code
hook_begin_jpeg(EB_Book*, EB_Appendix*, void *obj, 
		EB_Hook_Code, 
		int, const unsigned int* argv)
{
  QBookEntry *be = (QBookEntry *) obj;

  be->img_pos.page = argv[2];
  be->img_pos.offset = argv[3];
  be->img_type = IMG_JPEG;
  return EB_SUCCESS;
}

EB_Error_Code
hook_begin_bmp(EB_Book*, EB_Appendix*, void *obj, 
	       EB_Hook_Code, 
	       int, const unsigned int* argv)
{
  QBookEntry *be = (QBookEntry *) obj;

  be->img_pos.page = argv[2];
  be->img_pos.offset = argv[3];
  be->img_type = IMG_BMP;
  return EB_SUCCESS;
}

EB_Error_Code
hook_end_color(EB_Book* book, EB_Appendix*, void *obj, 
	       EB_Hook_Code, 
	       int, const unsigned int*)
{
  QBookEntry *be = (QBookEntry *) obj;

  if (be->img_type == IMG_NA) 
    return EB_SUCCESS;
  process_color(be, book, &be->img_pos, false);

  be->img_type = IMG_NA;


  return EB_SUCCESS;
}

EB_Error_Code
hook_end_in_color(EB_Book* book, EB_Appendix*, void *obj, 
		  EB_Hook_Code, 
		  int, const unsigned int*)
{
  QBookEntry *be = (QBookEntry *) obj;

  if (be->img_type == IMG_NA) 
    return EB_SUCCESS;
  process_color(be, book, &be->img_pos, true);
  be->img_type = IMG_NA;
  return EB_SUCCESS;
}

EB_Error_Code
hook_begin_mono(EB_Book*, EB_Appendix*, void *obj, 
		EB_Hook_Code, 
		int, const unsigned int* argv)
{
  QBookEntry *be = (QBookEntry *) obj;
  
  be->img_type = IMG_MONO;
  be->img_height = argv[2];
  be->img_width = argv[3];
  return EB_SUCCESS;
}

EB_Error_Code
hook_end_mono(EB_Book* book, EB_Appendix*, void *obj, 
	      EB_Hook_Code, 
	      int, const unsigned int* argv)
{
  QBookEntry *be = (QBookEntry *) obj;  
  if (be->img_type != IMG_MONO) 
    return EB_SUCCESS;
  EB_Position p;
  p.page = argv[1];
  p.offset = argv[2];
  if (eb_set_binary_mono_graphic(book, &p, be->img_width, be->img_height) 
      != EB_SUCCESS)
    {
      qDebug("eb_set_binary_mono_graphic failed");
      return EB_SUCCESS;
    }
  char *buf;
  int size = getBinary(book, &buf);

  QImage qim;
  qim.loadFromData((unsigned char*)buf, size);
  free(buf);

  char c[100]; // I guess this is enough

  int scale=config->ImageScale;
  if (config->ImageSizeLimitMode) {
    int sc0=100,sc1=100;
    if (qim.width()>config->MaxImageWidth) {
      sc0 = config->MaxImageWidth*100/qim.width();
    }
    if (qim.height()>config->MaxImageHeight) {
      sc1 = config->MaxImageHeight*100/qim.height();
    }
    if (sc0<scale) scale=sc0;
    if (sc1<scale) scale=sc1;
  }
  int w, h;
  w = (qim.width() * scale) / 100;
  h = (qim.height() * scale) / 100;

  if (scale<100) {
    int index = be->mmarray.size();
    be->mmarray.resize(index+1);
    be->mmarray[index].type = MM_IMAGE;
    be->mmarray[index].subType = be->img_type;
    be->mmarray[index].begin = p;
    be->mmarray[index].width = be->img_width;
    be->mmarray[index].height = be->img_height;

    sprintf (c, "%d_%d", LINK_IMAGE, be->mmarray.size()-1);
    be->addPositionString ( new QString (c) );
//    eb_write_text_string (book, "<p>");
    eb_write_text_string (book, INSERT_MARK);
    eb_write_text_string (book, "[Image]</a>");
//    sprintf(c, "<img source=\"image%d\"></p>", be->img_no);
    sprintf(c, "<img source=\"image%d\">", be->img_no);
    eb_write_text_string(book, c);
  } else {
//    sprintf(c, "<p><img source=\"image%d\"></p>", be->img_no);
    sprintf(c, "<img source=\"image%d\">", be->img_no);
    // A bug in qte? Original w/h makes an image displayed all black.
    eb_write_text_string(book, c);
  }
  sprintf(c, "image%d", be->img_no);
  if (scale==100) {
    QMimeSourceFactory::defaultFactory()->setImage(c, qim.convertDepth (COLOR_DEPTH));
  } else {
    QMimeSourceFactory::defaultFactory()->
      setImage(c, qim.convertDepth (COLOR_DEPTH).smoothScale(w,h));
  }
  //  QMimeSourceFactory::defaultFactory()->setImage(c, qim.convertDepth (COLOR_DEPTH));
  be->img_no++;
  be->img_type = IMG_NA;
  return EB_SUCCESS;
}

EB_Error_Code
hook_begin_wave(EB_Book* book, EB_Appendix*, void *obj, 
		EB_Hook_Code, 
		int, const unsigned int* argv)
{
  isInReference = true;
  QBookEntry *be = (QBookEntry *) obj;  
  
  eb_write_text_string (book, INSERT_MARK);

  //  char c[100]; // should be enough
  int index = be->mmarray.size();
  //  sprintf(c, "<a href=\"qbooksnd:%d\">", index);
  //  eb_write_text_string(book, c);

  be->mmarray.resize(index+1);
  be->mmarray[index].type=MM_SOUND;
  be->mmarray[index].subType=MM_SOUND_PCM;
  be->mmarray[index].begin.page = argv[2];
  be->mmarray[index].begin.offset = argv[3];
  be->mmarray[index].end.page = argv[4];
  be->mmarray[index].end.offset = argv[5];
  return EB_SUCCESS;
}

EB_Error_Code
hook_end_wave(EB_Book* book, EB_Appendix*, void* obj, 
	      EB_Hook_Code, 
	      int, const unsigned int*) {

  char c[32];
  QBookEntry *be = (QBookEntry *) obj;
  sprintf (c, "%d_%d", LINK_SOUND, be->mmarray.size()-1);
  be->addPositionString ( new QString (c) );

  eb_write_text_string(book, "</a>");
  isInReference = false;

  return EB_SUCCESS;
}

EB_Error_Code
hook_begin_mpeg(EB_Book* book, EB_Appendix*, void *obj, 
		EB_Hook_Code, 
		int, const unsigned int* argv)
{
  isInReference = true;
  QBookEntry *be = (QBookEntry *) obj;  
  
  eb_write_text_string (book, INSERT_MARK);
  eb_write_text_string (book, "[Movie]");

  int index = be->mmarray.size();

  be->mmarray.resize(index+1);
  be->mmarray[index].type=MM_MOVIE;
  be->mmarray[index].subType=0;
  be->mmarray[index].args[0] = argv[2];
  be->mmarray[index].args[1] = argv[3];
  be->mmarray[index].args[2] = argv[4];
  be->mmarray[index].args[3] = argv[5];

  return EB_SUCCESS;
}

EB_Error_Code
hook_end_mpeg(EB_Book* book, EB_Appendix*, void* obj, 
	      EB_Hook_Code, 
	      int, const unsigned int*)
{
  eb_write_text_string(book, "</a>");

  char c[32];
  QBookEntry *be = (QBookEntry *) obj;
  sprintf (c, "%d_%d", LINK_MOVIE, be->mmarray.size()-1);
  be->addPositionString ( new QString (c) );
  isInReference = false;

  return EB_SUCCESS;
}

#if 0
EB_Error_Code
hook_begin_xasound(EB_Book* book, EB_Appendix*, void *, 
		EB_Hook_Code, 
		int, const unsigned int*)
{
  fprintf(stderr,"Begin_XA_Sound\n");

  eb_write_text_string (book, INSERT_MARK);
  return EB_SUCCESS;
}

EB_Error_Code
hook_end_xasound(EB_Book* book, EB_Appendix*, void* obj, 
	      EB_Hook_Code, 
	      int, const unsigned int* argv)
{
  fprintf(stderr,"End_XA_Sound\n");

  QBookEntry *be = (QBookEntry *) obj;
  eb_write_text_string(book, "</a>");

  int index = be->mmarray.size();
  be->mmarray.resize(index+1);
  be->mmarray[index].type=MM_XASOUND;
  be->mmarray[index].subType=0;
  be->mmarray[index].args[0] = argv[1];
  be->mmarray[index].args[1] = argv[2];

  char c[32];
  sprintf (c, "%d_%d", LINK_XASOUND, be->mmarray.size()-1);
  be->addPositionString ( new QString (c) );

  return EB_SUCCESS;
}
#endif

int QBookEntry::fore_color;
int QBookEntry::back_color;

void
QBookEntry::openAppendix (QString appendix, EB_Subbook_Code sub_code)
{
  if (stop_code_mode!=1) return;
  if (!appendix) return;
  uint end = appendix.findRev (')');
  if (end > 0 && end == appendix.length () - 1)
    {
      int start = appendix.findRev ('(');
      if (start > 0)
	{
	  QString memory = appendix.left (end).right (end - start - 1);
	  QString name = (const char *) appendix.left (start);

	  const char *curd;
	  int i = 0;
	  for (curd = g_appendix_path[i].path; curd != NULL;
	       curd = g_appendix_path[++i].path)
	    {
	      if (strcmp (g_appendix_path[i].name, (const char *) memory) == 0)
		{
		  if (eb_bind_appendix (&(this->appendix),
					(const char *)
					(g_appendix_path[i].path + name)) !=
		      EB_SUCCESS)
		    {
		      return;
		    }
		  eb_set_appendix_subbook (&(this->appendix), sub_code);
		}
	    }
	}
    }
}

bool
QBookEntry::bindBook() {
  if (opened) unbindBook();
  opened = false;
  EB_Subbook_Code sub_codes[EB_MAX_SUBBOOKS];
  int sub_count;
  char ttl[EB_MAX_TITLE_LENGTH+1];

  if (DiskStat::enabled_p(memory)) {
    if (eb_bind (&book, (const char *) absolute_path) < 0) {
      return false;
    }

    current_codec=g_codec_euc;
    if (eb_character_code(&book, &ccode) == EB_SUCCESS) {
      if (ccode==EB_CHARCODE_ISO8859_1) {
	current_codec=g_codec_latin1;
	//======== Chinese, Korean, ...? ==========
      } 
    }

    eb_subbook_list (&book, sub_codes, &sub_count);
    if (sub_book_pos >= sub_count) {
      return false;
    }
    eb_subbook_title2 (&book, sub_codes[sub_book_pos], ttl);

    Utils::z2h(ttl);
    title=current_codec->toUnicode(ttl);

    eb_set_subbook (&book, sub_codes[sub_book_pos]);
    openAppendix (appendixPath, sub_codes[sub_book_pos]);
    eb_set_font (&book, EB_FONT_16);


    //    findMultiSearch();
    opened = true;
    return true;
  } 
  return false;
}

void
QBookEntry::unbindBook() {
  if (opened) {
    eb_finalize_book(&book);
  }
  opened=false;
}

QBookEntry::QBookEntry (QString ttl, QString path, int sub_bk_pos)
{
  int mx = config->MaxHits;
  if (mx < config->MultiMaxHits) mx = config->MultiMaxHits;
  if (mx < config->KeywordMaxHits) mx = config->KeywordMaxHits;
  hits = new EB_Hit[mx];
  stop_code0=stop_code1=stop_code_mode=0;
  EB_Hook heading_hooks [] = {
    {EB_HOOK_NARROW_FONT, text_hook_narrow_font}, 
    {EB_HOOK_NEWLINE, text_hook_null},
    {EB_HOOK_NARROW_JISX0208, text_hook_narrow_jisx0208},
    {EB_HOOK_WIDE_FONT, text_hook_wide_font},
    {EB_HOOK_GB2312, text_hook_gb2312},

    {EB_HOOK_BEGIN_SUBSCRIPT, text_hook_begin_subscript},
    {EB_HOOK_END_SUBSCRIPT, text_hook_end_subscript},
    {EB_HOOK_BEGIN_SUPERSCRIPT, text_hook_begin_superscript},
    {EB_HOOK_END_SUPERSCRIPT, text_hook_end_superscript},

    {EB_HOOK_NULL, NULL}
  };

  EB_Hook text_hooks [] = {
    {EB_HOOK_NARROW_FONT, text_hook_narrow_font}, 
    {EB_HOOK_NEWLINE, text_hook_newline},
    {EB_HOOK_WIDE_FONT, text_hook_wide_font}, 
    {EB_HOOK_NARROW_JISX0208, text_hook_narrow_jisx0208}, 
    {EB_HOOK_GB2312, text_hook_gb2312},

    {EB_HOOK_BEGIN_REFERENCE, text_hook_begin_reference}, 
    {EB_HOOK_END_REFERENCE, text_hook_end_reference}, 
    {EB_HOOK_GRAPHIC_REFERENCE, text_hook_graphic_reference}, 
    {EB_HOOK_BEGIN_GRAPHIC_REFERENCE, text_hook_begin_image_group}, 
    {EB_HOOK_END_GRAPHIC_REFERENCE, text_hook_end_image_group}, 
    {EB_HOOK_BEGIN_CANDIDATE, text_hook_begin_reference},
    {EB_HOOK_END_CANDIDATE_LEAF, text_hook_end_reference},
    {EB_HOOK_END_CANDIDATE_GROUP, text_hook_end_reference},

    {EB_HOOK_BEGIN_SUBSCRIPT, text_hook_begin_subscript},
    {EB_HOOK_END_SUBSCRIPT, text_hook_end_subscript},
    {EB_HOOK_BEGIN_SUPERSCRIPT, text_hook_begin_superscript},
    {EB_HOOK_END_SUPERSCRIPT, text_hook_end_superscript},
    {EB_HOOK_BEGIN_DECORATION, text_hook_begin_decoration},
    {EB_HOOK_END_DECORATION, text_hook_end_decoration},
    {EB_HOOK_BEGIN_EMPHASIS, text_hook_begin_emphasis},
    {EB_HOOK_END_EMPHASIS, text_hook_end_emphasis},
    {EB_HOOK_BEGIN_KEYWORD, text_hook_begin_keyword},
    {EB_HOOK_END_KEYWORD, text_hook_end_keyword},

    {EB_HOOK_BEGIN_COLOR_JPEG, hook_begin_jpeg},
    {EB_HOOK_BEGIN_COLOR_BMP, hook_begin_bmp},
    {EB_HOOK_BEGIN_IN_COLOR_JPEG, hook_begin_jpeg},
    {EB_HOOK_BEGIN_IN_COLOR_BMP, hook_begin_bmp},
    {EB_HOOK_END_COLOR_GRAPHIC, hook_end_color},
    {EB_HOOK_END_IN_COLOR_GRAPHIC, hook_end_in_color},

    {EB_HOOK_BEGIN_MONO_GRAPHIC, hook_begin_mono}, 
    {EB_HOOK_END_MONO_GRAPHIC, hook_end_mono},

    {EB_HOOK_BEGIN_WAVE, hook_begin_wave}, 
    {EB_HOOK_END_WAVE, hook_end_wave},
    {EB_HOOK_BEGIN_MPEG, hook_begin_mpeg}, 
    {EB_HOOK_END_MPEG, hook_end_mpeg},

//    {ZTEN_HOOK_BEGIN_EBXA_SOUND, hook_begin_xasound}, 
//    {ZTEN_HOOK_END_EBXA_SOUND, hook_end_xasound},

    {EB_HOOK_INITIALIZE, hook_initialize},

    {EB_HOOK_NULL, NULL}
  };

  current_codec=g_codec_euc;

  opened = false;

  memory = DiskStat::memory(path);

//  eb_initialize_hookset(&zten_default_hookset);

  eb_initialize_hookset(&heading_hookset);
  eb_set_hooks(&heading_hookset, heading_hooks);

  eb_initialize_hookset(&text_hookset);
  eb_set_hooks(&text_hookset, text_hooks);  

  eb_initialize_book(&book);
  eb_initialize_appendix(&appendix);

  pos_list.setAutoDelete (TRUE);

  absolute_path = path;
  sub_book_pos = sub_bk_pos;
  title = ttl;
 
  menu_enabled=false;
  alt_title=title;
  alt_title_color="#000000";
  
  for(int i=0;i<MAX_DIC_GROUPS+1;i++) {
    priority[i]=0;
  }
  priority[DIC_GROUP_ALL]=1;

  if (!bindBook()) return;

  kanaConv=(book.disc_code==EB_DISC_EB)?true:false;

}

bool
QBookEntry::isOpened ()
{
  return opened;
}

bool
QBookEntry::isBound ()
{
  if (!isOpened()) return false;
  return eb_is_bound(&book)==EB_SUCCESS;
}

bool QBookEntry::isReady() 
{
  return DiskStat::enabled_p(memory);
}

void
QBookEntry::findMultiSearch (bool all)
{ 
  EB_Multi_Search_Code multi_list[EB_MAX_MULTI_SEARCHES];
  int entry_count;
  char multi_title [EB_MAX_MULTI_TITLE_LENGTH+1];
  int i;
  
  if (eb_multi_search_list (&book, multi_list, 
			    &n_multi_searches) != EB_SUCCESS) {
    n_multi_searches = 0;  
    return;
  }
  //  qDebug ("Multi search for %s", title);

  for (i = 0; i < n_multi_searches; i++) {
    if (!all && !multi_search_enabled[i] && group!=DIC_GROUP_SINGLE) continue;
    if (eb_multi_entry_count (&book, multi_list[i], &entry_count)
	== EB_SUCCESS) {
      multi_title[0] = '\0';
      eb_multi_title (&book, multi_list[i], multi_title);
      if (multi_search_alt_title[i]=="") {
	multi_search_alt_title[i]=
	  QString (current_codec->toUnicode(multi_title));
      }
      emit foundMultiSearch (this, multi_list[i],
			     multi_search_alt_title[i], entry_count);
    }
  }
  for(; i<EB_MAX_MULTI_SEARCHES;i++) {
    multi_search_enabled[i]=false;
  }
  
  return;
}

void
QBookEntry::findMultiSearchEntry (int multi_id)
{
  int entry_count;
  char entry_label [EB_MAX_MULTI_LABEL_LENGTH + 1];

  if (eb_multi_entry_count (&book, multi_id, &entry_count)
      == EB_SUCCESS)
    {
      for (int i = 0; i < entry_count; i++)
	{
	  if (eb_multi_entry_label (&book, multi_id, i, entry_label)
	      == EB_SUCCESS)
	    {
	      emit foundMultiSearchEntry (this, i,
					  QString(current_codec->
						  toUnicode(entry_label)),
					  eb_multi_entry_have_candidates
					  (&book, multi_id, i));
	    }
	}
    }
}

int
QBookEntry::multiSearch (EB_Multi_Search_Code code, QList<QString> keywords)
{
  QCString cword;
  char **multi = NULL;
  int i;

  multi = (char **) malloc (sizeof (const char *) * (keywords.count () + 1));
  for (i = 0; i < (int) keywords.count (); i++)
    {
      // cword = g_codec->fromUnicode(*keywords.at(i));
      // multi[i] = strdup ((const char *) cword);
      multi[i] = strdup ((const char *) *keywords.at (i));
    }
  multi[i] = NULL;
  eb_search_multi (&book, code, (const char **) multi);
  for (i = 0; i < (int) keywords.count (); i++)
    {
      free ((void *) multi[i]);
    }
  free (multi);
  eb_hit_list (&book, config->MultiMaxHits, hits, &hit_count);
  return hit_count;
}

EB_Error_Code QBookEntry::search_keyword(int match_type, EB_Book *book,
					 const char * const input_words[]) {
  EB_Search backup;
  EB_Position p;
  EB_Error_Code e;

  if (match_type == CROSSWORD_MATCH) {
    if (crossSearchPosition(&p)) {
      memcpy(&backup, &book->subbook_current->keyword, sizeof(EB_Search));
      book->subbook_current->keyword.start_page=p.page;
      e = eb_search_keyword(book, input_words);
      memcpy(&book->subbook_current->keyword, &backup, sizeof(EB_Search));
      return e==EB_SUCCESS;
    } else {
      return false;
    }
  } else {
    e = eb_search_keyword(book, input_words);
  }
  return e==EB_SUCCESS;
}



int
QBookEntry::search (QString keyword, int match_type)
{
  const char *word = NULL;
  char *word2 = NULL;
  // for keyword match.
  const char **keywords = NULL;
  QCString cword;
  int result_size = 0;
  bool search_result=false;

  if (!isReady()) return -1;
  if (!isOpened()) {
    if (!bindBook()) return -1;
  }

  if (keyword.length() == 0) return 0;

  if (keyword.length() > 1) {
    if (keyword.left(1) == "'" || keyword.left(1) == tr("'")) {
      return search(keyword.mid(1), KEYWORD_MATCH);
    } else if (keyword.left(1) == "/" || keyword.left(1) == tr("/")) {
      return search(keyword.mid(1), CROSSWORD_MATCH);
    } else if (keyword.left(1) == "\"" || keyword.left(1) == tr("\"")) {
      return search(keyword.mid(1), EXACT_MATCH);
    } else if (keyword.right(1) == "\"" || keyword.right(1) == tr("\"")) {
      return search(keyword.left(keyword.length()-1), EXACT_MATCH);
    } else if (keyword.left(1) == "*" || keyword.left(1) == tr("*")) {
      return search(keyword.mid(1), BACKWARD_MATCH);
    } else if (keyword.right(1) == "*" || keyword.right(1) == tr("*")) {
      return search(keyword.left(keyword.length()-1), FORWARD_MATCH);
    }
  }

  result_size = config->MaxHits;
  cword = g_codec_euc->fromUnicode(keyword.replace(tr("&nbsp;").left(1)," "));
  word = cword;
  //  fprintf(stderr,"word='%s'\n",word);

  switch (match_type)
    {
    case FORWARD_MATCH:
      search_result=(eb_search_word (&book, word) == EB_SUCCESS);
      break;
    case BACKWARD_MATCH:
      search_result=(eb_search_endword (&book, word) == EB_SUCCESS);
      break;
    case EXACT_MATCH:
      search_result=(eb_search_exactword (&book, word) == EB_SUCCESS);
      break;
    case KEYWORD_MATCH:
    case CROSSWORD_MATCH:
      QStringList strlist = QStringList::split (" ", word);
      int count = strlist.count ();
      keywords = (const char **) malloc (sizeof (const char *) * (count + 1));
      int i;
      for (i = 0; i < count; i++) {
	keywords[i] = strdup ((const char *) strlist[i]);
      }
      keywords[i] = NULL;
      result_size = config->KeywordMaxHits;
      search_result=search_keyword (match_type, &book, keywords);
      for (i = 0; i < count - 1; i++) {
	free ((void *) keywords[i]);
      }
      free (keywords);
      break;
    }

  if (search_result) {
    eb_hit_list (&book, result_size, hits, &hit_count);
  } else {
    hit_count=0;
  }

  if (hit_count == 0 && kanaConv && z_kakasi_do!=NULL && z_kakasi_free!=NULL)
    {
      search_result=false;
      switch (match_type)
	{
	case FORWARD_MATCH:
	  word2 = z_kakasi_do ((char *)word);
	  search_result=(eb_search_word (&book, word2) == EB_SUCCESS);
	  z_kakasi_free (word2);
	  break;
	case BACKWARD_MATCH:
	  word2 = z_kakasi_do ((char *)word);
	  search_result=(eb_search_endword (&book, word2) == EB_SUCCESS);
	  z_kakasi_free (word2);
	  break;
	case EXACT_MATCH:
	  word2 = z_kakasi_do ((char *)word);
	  search_result=(eb_search_exactword (&book, word2) == EB_SUCCESS);
	  z_kakasi_free (word2);
	  break;
	case KEYWORD_MATCH:
	case CROSSWORD_MATCH:
	  QStringList strlist = QStringList::split (" ", word);
	  int count = strlist.count ();
	  keywords = (const char **) malloc
	    (sizeof (const char *) * (count + 1));
	  int i;
	  for (i = 0; i < count; i++) {
	      keywords[i] = (const char *)
		z_kakasi_do ((char *)(const char *) strlist[i]);
	  }
	  keywords[i] = NULL;
	  search_result=search_keyword (match_type, &book, keywords);
	  for (i = 0; i < count; i++) {
	    z_kakasi_free ((char *) keywords[i]);
	  }
	  free (keywords);
	  break;
	}
      if (search_result) {
	eb_hit_list (&book, result_size, hits, &hit_count);
      } else {
	hit_count=false;
      }
    }
  return hit_count;
}

void
QBookEntry::addPositionString (QString *str)
{
  pos_list.append (str);
}

void
QBookEntry::playSound (int index)
{
  if (eb_set_binary_wave(&book, &mmarray[index].begin,
			 &mmarray[index].end) != EB_SUCCESS)
    {
      qDebug("eb_set_binary_wave failed");
      return;
    }
  char *buf;
  int size = getBinary(&book, &buf);
#ifdef MAC
  QFile f(QBOOK_TEMP_WAV);
  f.open(IO_WriteOnly);
  QDataStream ds(&f);
  ds.writeRawBytes(buf, size);
  free(buf);
  QSound::play(QBOOK_TEMP_WAV);
  // f.remove (); QSound::play returns immediately.
#else
  /*
 #define QBOOK_TEMP_WAV "/home/tmp/ztenv.wav"
  QFile f(QBOOK_TEMP_WAV);
  f.open(IO_WriteOnly);
  QDataStream ds(&f);
  ds.writeRawBytes(buf, size);
  */

  PlaySound(buf,size);
  free(buf);
#endif
}


#if 0
void
QBookEntry::playXASound (int index)
{

  fprintf(stderr,"XA Sound: %d-%d :\n",
	  mmarray[index].args[0],mmarray[index].args[1]);
  
  char path[EB_MAX_PATH_LENGTH+1],filename[EB_MAX_PATH_LENGTH+1];
  if (eb_find_file_name2(book.path, book.subbook_current->directory_name,
			 "sound", filename) == EB_SUCCESS) {
    snprintf(path,EB_MAX_PATH_LENGTH,"%s/%s/%s",
	     book.path, book.subbook_current->directory_name, filename);
    fprintf(stderr,"Path=%s\n", path);
    PlayXASound(path, mmarray[index].args[0],mmarray[index].args[1]);
  }
}
#endif


#if 0
static const char* MovieFileName(const unsigned int *argv) {
  static char fname[9];
  const char table21[]=
    " ??,.?:;?!?????"
    "^~_????????" "?---/"
    "\\~?|??`'\"\"()[][]"
    "{}<>????????+-??"
    "?=?<>???????" "'\"?\\"
    "$??%#&*@???????";
  unsigned char jh[8],jl[8];
  for(int i=0;i<4;i++){
    jh[i*2]  = 0xff & (argv[i]>>24);
    jl[i*2]  = 0xff & (argv[i]>>16);
    jh[i*2+1]= 0xff & (argv[i]>>8);
    jl[i*2+1]= 0xff & argv[i];
  }
  for(int i=0;i<8;i++){
    if (jh[i] == 0x21) {
      if (jl[i]==0x21) {
	fname[i]=0;
	break;
      } else if (0x21<jl[i] && jl[i]<0x7f && table21[jl[i]-0x21]!='?') {
	fname[i]=table21[jl[i]-0x21];
      } else {
	return NULL;
      }
    }  else if (jh[i]==0x23) {
      if ((0x30<=jl[i] && jl[i]<=0x39) || 
	  (0x41<=jl[i] && jl[i]<=0x5a) ||
	  (0x61<=jl[i] && jl[i]<=0x7a)) {
	fname[i]=jl[i];
      } else {
	return NULL;
      }
    }
  }
  fname[8]=0;
  return fname;
}
#endif

void
QBookEntry::playMovie (int index)
#if 0
{
  static char* exts[]={
    "",
    ".mpg",
    ".asf",
    ".mp4",
    ".mov",
    ".3gpp",
    NULL,
  };
  static char linkname[]="/tmp/ztenv.mpg";
  static char decodedname[]="/home/tmp/ztenv.mpg";
  const char *fname;
  char
    path[EB_MAX_PATH_LENGTH+1],
    fnameex[EB_MAX_PATH_LENGTH+1],
    filename[EB_MAX_PATH_LENGTH+1], 
    cmd[1024];

  fname=MovieFileName((const unsigned int *)mmarray[index].args);
  if (fname==NULL) return;

  for(int i=0; exts[i]!=NULL; i++) {
    struct stat st;
    snprintf(fnameex,EB_MAX_PATH_LENGTH,"%s%s",fname,exts[i]);
    if (eb_find_file_name3(book.path, book.subbook_current->directory_name,
			   book.subbook_current->movie_directory_name,
			   fnameex, filename) == EB_SUCCESS) {
      char *moviefile;
      snprintf(path,EB_MAX_PATH_LENGTH,"%s/%s/%s/%s",
	       book.path, 
	       book.subbook_current->directory_name,
	       book.subbook_current->movie_directory_name,
	       filename);
      if (i==0) {
	if (lstat(linkname,&st)>=0) {
	  unlink(linkname);
	}
	symlink(path,linkname);
	moviefile=linkname;
      } else {
	moviefile=path;
      }

      bool c3000=false;
      FILE* fp=fopen(moviefile,"r");
      if (fp!=NULL) {
	char buf[128];
	if (fread(buf,1,128,fp)==128){
	  if (buf[0]==0 && buf[1]==0 && buf[16]==0x53 && buf[17]==0x68)
	    c3000=true;
	}
	fclose(fp);
      }

      if (c3000) {
	void* dlhandle=dlopen("/opt/QtPalmtop/lib/libzsecurefile.so",
			      RTLD_NOW);
	if (dlhandle!=NULL) {
	  zz_decode_file =
	    (int (*)(const char *, const char *)) 
	    dlsym(dlhandle, "z_decode_file");
	  if (zz_decode_file!=NULL) {
	    unlink(decodedname);
	    int z_error=zz_decode_file(moviefile, decodedname);
	    if (z_error == 0) {
	    moviefile=decodedname;
	    }
	  }
	  dlclose(dlhandle);
	}
      }
      sprintf(cmd,(const char *)config->MovieCommand,moviefile);
      system(cmd);
      break;
    }
  }
  return;
}
#else
{
  if(eb_set_binary_mpeg(&book, (const unsigned int *)mmarray[index].args) != EB_SUCCESS)
  {
    qDebug("eb_set_binary_mpeg failed");
    return;
  }

  static char moviefile[]="/home/tmp/ztenv.mpg";
  char *buf;
  int size = getBinary(&book, &buf);
  unlink(moviefile);
  FILE *fp = fopen(moviefile, "w");
  fwrite(buf,size,1,fp);
  fclose(fp);
  free(buf);

  char cmd[1024];
  sprintf(cmd,(const char *)config->MovieCommand,moviefile);
  system(cmd);
}
#endif

void
QBookEntry::showImage (int index)
{
  QImage qim;
  ZtenImage b (qbb, 0, true);
  QString caption="";

  switch (mmarray[index].subType) {
  case IMG_JPEG:
  case IMG_BMP:
    {
      if (eb_set_binary_color_graphic(&book, 
				      &mmarray[index].begin) != EB_SUCCESS) {
	qDebug("eb_set_binary_color_graphic failed");
	return;
      }
      char *buf;
      int size;
      EB_Disc_Code disc_code;

      if (mmarray[index].type == IMG_JPEG &&
	  (eb_disc_type(&book, &disc_code) != EB_SUCCESS ||
	   disc_code != EB_DISC_EPWING))
	size = getJpegBinary(&book, &buf);
      else 
	size = getBinary(&book, &buf);

      qim.loadFromData((unsigned char*)buf, size);
      free(buf);
    }
    break;
  case IMG_MONO:
    {
      if (eb_set_binary_mono_graphic(&book, &mmarray[index].begin, 
				     mmarray[index].width,
				     mmarray[index].height)
	  != EB_SUCCESS) {
	qDebug("eb_set_binary_mono_graphic failed");
	return;
      }
      char *buf;
      int size = getBinary(&book, &buf);
      qim.loadFromData((unsigned char*)buf, size);
      free(buf);
    }
    break;
  default:
    caption="Image Decode Error";
    break;
  }

  b.setImage(qim,caption);
  b.showMaximized ();
  b.exec ();  
}

bool
QBookEntry::showImageGroup (int index, EB_Position& pos)
{

  pos=mmarray[index].begin;
  return showImageGroup(pos);
}

bool
QBookEntry::showImageGroup (EB_Position& pos) 
{
  const int bufSize=2048;
  char readbuf[bufSize];
  char* buf=readbuf;
  int len;
  //  EB_Position p;

  eb_seek_text(&book, &pos);
  eb_read_rawtext(&book,bufSize,buf,&len);
  if (len<10) return false;
  if (buf[0]==0x1f && buf[1]==0x4b && buf[8]==0x1f && buf[9]==0x6b) {
    pos.page=getBCD8(buf+2); 
    pos.offset=getBCD4(buf+6);
    //    emit goTo(p);
    return true;
  } else if (buf[0]==0x1f && buf[1]==0x4c && buf[2]==0 && buf[3]==1) {
    buf+=4;
    len-=4;

    QImage qim;
    ZtenImage b (qbb, 0, true);
    QString caption="";

    while(len>=36) {
      if (buf[0]!=0x1f) break;
      if (buf[0]==0x1f && buf[1]==0x6c) break;
      switch (buf[1]){
      case 0x4d:
	pos.page=getBCD8(buf+14);
	pos.offset=getBCD4(buf+18);
	if (eb_set_binary_color_graphic(&book, &pos) == EB_SUCCESS) {
	  char *gbuf;
	  int size;
	  size = getBinary(&book, &gbuf);
	  qim.loadFromData((unsigned char*)gbuf, size);
	  free(gbuf);
	  b.setImage(qim,"");
	}
	buf+=22;
	len-=22;
	break;

      case 0x4f:
	{
	  int x=getBCD4(buf+0x08);
	  int y=getBCD4(buf+0x0a);
	  int w=getBCD4(buf+0x0c);
	  int h=getBCD4(buf+0x0e);
	  int page=getBCD8(buf+0x1c);
	  int offset=getBCD4(buf+0x20);
	  b.addArea(x,y,x+w,y+h,page,offset);
	  buf+=36;
	  len-=36;
	}
	break;

      default:
	break;
      }
    }
    b.showMaximized ();
    if (b.exec ()==1) {
      pos.page=b.page();
      pos.offset=b.offset();
      return true;
    }
  }
  return false;
}

QString
QBookEntry::getTitle ()
{
  return title;
  /*
  char *t = strdup(title);
  Utils::z2h(t);
  QString s(g_codec_euc->toUnicode(t));
  free(t);
  return s;
  */
}

void 
QBookEntry::setAltTitle(QString at, QString atc)
{
  alt_title = at;
  alt_title_color = atc;
}

QString
QBookEntry::getAltTitle ()
{
  return alt_title;
}

QString
QBookEntry::getAltTitleColor ()
{
  return alt_title_color;
}

bool
QBookEntry::isRedundantEntry (int hit_pos)
{
  int page = hits[hit_pos].text.page;
  int offset = hits[hit_pos].text.offset;
  // If same page exists earlier, it is redundant entry.
  for (int i = hit_pos - 1; i >= 0; i--)
    {
      if (hits[i].text.page == page &&
	  hits[i].text.offset == offset)
	return true;
    }
  return false;
}
  
QString
QBookEntry::getHeading (int hit_pos)
{
  QString s;
  char text[MAX_HEADING+1];
  int text_length;
  EB_Position p0;
  last_wide_font = last_narrow_font = 0;
  UseSubSupFont=config->UseSubSupFont;
  
  if (eb_tell_text (&book, &p0) != EB_SUCCESS)
    p0.page = -1;
  if (eb_seek_text (&book, &hits[hit_pos].heading) != EB_SUCCESS)
    return QString (); // isNull.
  int bytes = 0;

  while (1)
    {
      if (eb_read_heading(&book, NULL, &heading_hookset, this,
			  MAX_HEADING, text, &text_length) != EB_SUCCESS)
	{
	  bytes = 0;
	  break;
	}
      bytes += text_length;
      if (text_length < 1)
	break;
      s.append(text);
    }
  if (bytes == 0)
    {
      if (p0.page >= 0)
	eb_seek_text(&book, &p0);
      return QString ();
    }
  // Remove redundant entries.
  if (isRedundantEntry (hit_pos))
    {
      return QString (); // return Null String.
    }
  else
    {
      if (ccode==EB_CHARCODE_JISX0208_GB2312) {
	int point;
	s=current_codec->toUnicode(s);
	while ((point = s.find (GB2312_MARK, 0, FALSE)) >= 0) {
	  QString str;
	  bool ok;
	  char buf[3];
	  str=s.mid(point+10,2);
	  buf[0]=str.toInt(&ok,16);
	  str=s.mid(point+12,2);
	  buf[1]=str.toInt(&ok,16);
	  buf[2]=0;
	  str=g_codec_gbk->toUnicode(buf);
	  s.replace((unsigned int) point, 14, str); // "__GB2312__xxxx "
	}
	return s;
      } else {
	return current_codec->toUnicode(s);
      }
    }
}

QString
QBookEntry::getIdentity ()
{
  char s[3];
  sprintf (s, "%d", sub_book_pos);
  return absolute_path + ":" + s;
}

bool
QBookEntry::equals (QString identity)
{
  char s[3];
  sprintf (s, "%d", sub_book_pos);
  return absolute_path + ":" + s == identity;
}


EB_Position
QBookEntry::getHitPosition (int hit_pos)
{
  return hits[hit_pos].text;
}


// Ignore Ctrl-X,C,V, Fn-Shift-C (Cut/Copy/Paste/ScreenCapture)
static struct LinkKeyStruct {
  char ch;
  int keycode;
  int code;
} LinkKeys[]={
  //  {'0', Qt::Key_0,0},
  {'1', Qt::Key_1,1},
  {'2', Qt::Key_2,2},
  {'3', Qt::Key_3,3},
  {'4', Qt::Key_4,4},
  {'5', Qt::Key_5,5},
  {'6', Qt::Key_6,6},
  {'7', Qt::Key_7,7},
  {'8', Qt::Key_8,8},
  {'9', Qt::Key_9,9},

  {'A', Qt::Key_A,10},
  {'B', Qt::Key_B,11},
  //  {'C', Qt::Key_C,12},
  {'D', Qt::Key_D,13},
  {'E', Qt::Key_E,14},
  {'F', Qt::Key_F,15},
  {'G', Qt::Key_G,16},
  {'H', Qt::Key_H,17},
  {'I', Qt::Key_I,18},
  {'J', Qt::Key_J,19},
  {'K', Qt::Key_K,20},
  {'L', Qt::Key_L,21},
  {'M', Qt::Key_M,22},
  {'N', Qt::Key_N,23},
  {'O', Qt::Key_O,24},
  {'P', Qt::Key_P,25},
  {'Q', Qt::Key_Q,26},
  {'R', Qt::Key_R,27},
  {'S', Qt::Key_S,28},
  {'T', Qt::Key_T,29},
  {'U', Qt::Key_U,30},
  //  {'V', Qt::Key_V,31},
  {'W', Qt::Key_W,32},
  //  {'X', Qt::Key_X,33},
  {'Y', Qt::Key_Y,34},
  {'Z', Qt::Key_Z,35},
};

EB_Position
QBookEntry::getLinkPosition (int key)
{
  EB_Position p;
  p.page = 0;
  p.offset = -1;
  unsigned int link_pos = pos_list.count();

  if (key>=0) {
    for(unsigned int i=0; i < sizeof(LinkKeys)/sizeof(LinkKeyStruct); i++) {
      if (LinkKeys[i].keycode==key) {
	link_pos = i;
	break;
      }
    }
  } else {
    key=-key;
    if (key<36) {
      for(unsigned int i=0; i < sizeof(LinkKeys)/sizeof(LinkKeyStruct); i++) {
	if (LinkKeys[i].code==key) {
	  link_pos = i;
	  break;
	}
      }
    } else {
      link_pos=key-36+sizeof(LinkKeys)/sizeof(LinkKeyStruct);
    }
  }


  if (link_pos < pos_list.count ())
    {
      int page, offset;
      QString *pos = pos_list.at(link_pos);
      sscanf(pos->latin1(), "%d_%d", &page, &offset);
      p.page = page;
      p.offset = offset;
      return p;
    }
  return p;
}

QString
QBookEntry::getText (EB_Position pos, QBookBrowser* qb)
{
  QString s;
  char text[MAX_TEXT+1];
  int text_length;
  EB_Position p0;

  zten_stop_code0 = stop_code0;
  zten_stop_code1 = stop_code1;

  last_wide_font = last_narrow_font = 0;
  refCount=0;

  qbb=qb;
  UseSubSupFont=config->UseSubSupFont;

  pos_list.clear ();
  img_type = IMG_NA;
  img_no = 0;
  if (mmarray.size () > 0)
    {
      mmarray.truncate (0);
    }

  if (eb_tell_text (&book, &p0) != EB_SUCCESS)
    p0.page = -1;

  QArray<EB_Position> localHistory;
  for(;;) {
    localHistory.resize(localHistory.size()+1);
    localHistory[localHistory.size()-1]=pos;
    //    zten->addHistory(pos,this);
    char buf[10];
    int len;
    if (eb_seek_text (&book, &pos) != EB_SUCCESS)
      return QString ();
    eb_read_rawtext(&book,10,buf,&len);
    if (buf[0]==0x1f && ((buf[1]==0x4b && buf[9]==0x6b) || buf[1]==0x4c)) {
      if (showImageGroup (pos)) {
	fprintf(stderr,"goto %d:%d\n", pos.page, pos.offset);
      } else {
	if (localHistory.size()<=1) return QString();
	pos=localHistory[localHistory.size()-2];
	localHistory.resize(localHistory.size()-2);
      }
    } else {
      break;
    }
  } 

  if (eb_seek_text (&book, &pos) != EB_SUCCESS)
    return QString ();

  int bytes = 0;
  EB_Appendix *appdx = NULL;
  if (eb_is_appendix_bound (&appendix)) {
    appdx = &appendix;
  }
  in_subsup=false;
  n_decoration=0;

  bool isContinuedPage = false;
  EB_Error_Code err;
  err = eb_forward_text(&book, appdx);
  if(err != EB_ERR_END_OF_CONTENT) {
    if(err != EB_SUCCESS) {
      return QString();
    } else {
      err = eb_backward_text(&book, appdx);
      if(err != EB_SUCCESS) {
        return QString ();
      }
      EB_Position temp;
      err = eb_tell_text(&book, &temp);
      if(err != EB_SUCCESS) {
        return QString ();
      }
      if(temp.page == pos.page   && abs(temp.offset- pos.offset      ) <= 4
      || temp.page == pos.page+1 && abs(temp.offset-(pos.offset-2048)) <= 4
      || temp.page == pos.page-1 && abs(temp.offset-(pos.offset+2048)) <= 4) {
        isContinuedPage = false;
/*
fprintf(stderr,
	"new page ?\n"
	"temp:page=%d:offset=%d\n"
	"pos :page=%d:offset=%d\n",
	temp.page,temp.offset,
	pos.page,pos.offset);
*/
	pos = temp;
      } else {
        isContinuedPage = true;
/*
fprintf(stderr,
	"continue page ?\n"
	"temp:page=%d:offset=%d\n"
	"pos :page=%d:offset=%d\n",
	temp.page,temp.offset,
	pos.page,pos.offset);
*/
      }
    }
  }
  err = eb_seek_text(&book, &pos);
  if(err != EB_SUCCESS) {
    return QString ();
  }
/*
fprintf(stderr,
	"pos :page=%d:offset=%d\n",
	pos.page,pos.offset);
*/

  //  app->setOverrideCursor(WaitCursor);

  QProgressBar *p = NULL;
  if (config->EnableProgressBar) {
    p=new QProgressBar(config->MaxContentLength,qbb);
    p->move(10,200);
    p->resize(200,40);
    p->show();
    p->reset();
  }

  int max_len=config->MaxContentLength;
  int count=1;
  if (stop_code_mode==2 /*|| stop_code_mode==3*/) {
    // ================ ToDo =======================
    max_len*=5;
    count=50;
  }
  bool text_limit_flag=false;
  while(count>0) {
    while (1) {
      EB_Error_Code err;
      if (config->EnableProgressBar) {
	p->setProgress(bytes);
      }
      //      app->processOneEvent();
      if ((err=eb_read_text(&book, appdx, &text_hookset, this,
		       MAX_TEXT, text, &text_length)) != EB_SUCCESS) {
fprintf(stderr,"eb_read_text\n%s\n%s\n",eb_error_string(err),eb_error_message(err));
	bytes = 0;
	break;
      }
      if (text_length < 1) 
	break;
      if(stop_code_mode == 3 && isContinuedPage && isHookKeyword) {
        QString temp(text);
        temp.replace(QRegExp("<strong>.*$"), "");
        bytes += temp.length();
        s.append(temp);
        count = 0;
        break;
      } else {
        bytes += text_length;
        s.append(text);
      }
      if(eb_is_text_stopped(&book) != 0) {
        break;
      }
      if (!isInReference && bytes>=max_len) {
	count=0; 
	text_limit_flag=true;
	break;
      }
    }
    count--;
    if (count>0) {
      if(eb_forward_text(&book, appdx) != EB_SUCCESS)
	break;
    }
  }
  if (text_limit_flag) {
    char buf[40];
    s.append("<br>\n ");
    s.append(INSERT_MARK);
    s.append("[[More]]");
    sprintf(buf,"%d_%d",last_newline.page,last_newline.offset);
    pos_list.append(new QString(buf));
    pos_list.append(new QString(buf));
    s.append("</a>");
  }
  s.prepend("<html><body>");
  s.append("</body></html>");

  if (config->EnableProgressBar) {
    p->setProgress(config->MaxContentLength);
    delete p;
  }

  //  app->restoreOverrideCursor();
  if (bytes == 0)
    {
      if (p0.page >= 0)
	eb_seek_text(&book, &p0);
      //      s=QString("");
      if (opened){
	s.append(QString("\n\nRead Error\n"));
      } else {
	s.append(QString("\n\nNot Opened\n"));
      }
      return s;
    }
  int point = 0;
  unsigned int position = 0;
  QString *ref;

  //  char height[5];
  //  sprintf (height, "%d", getFontSize ());

  // #ifndef ZAURUS
#if 1
  QFile f1("/tmp/test1.html");
  f1.open(IO_WriteOnly);
  QDataStream ds1(&f1);
  ds1.writeRawBytes((const char *)s, s.length());
#endif


  unsigned int s1=sizeof(LinkKeys)/sizeof(LinkKeyStruct);
  while ((point = s.find (INSERT_MARK, 0, FALSE)) >= 0) {
    QString str;
    ref = pos_list.at (position++);
    str = QString ("<a href=\"qbook:") + (const char *)*ref + "\">";

    if (position <= s1) {
      str.append(QString("<font color=\"#ff0000\"><b>[#%1]</b></font>").
		 arg(LinkKeys[position-1].ch));
    } else {
      str.append(QString("<font color=\"#ff0000\"><b>[%1]</b></font>").
		 arg(position-1-s1+36,2,36));
    }
    s.replace((unsigned int) point, 1, str);
  }

#ifdef MAC
  s=s.replace("\xa1\xa1","&nbsp;&nbsp;");  // zenkaku space => hankaku space * 2
#endif

  // #ifndef ZAURUS
#if 1
  QFile f("/tmp/test.html");
  f.open(IO_WriteOnly);
  QDataStream ds(&f);
  ds.writeRawBytes((const char *)s, s.length());
#endif

  if (ccode==EB_CHARCODE_JISX0208_GB2312) {
    s=current_codec->toUnicode(s);
    while ((point = s.find (GB2312_MARK, 0, FALSE)) >= 0) {
      QString str;
      bool ok;
      char buf[3];
      str=s.mid(point+10,2);
      buf[0]=str.toInt(&ok,16);
      str=s.mid(point+12,2);
      buf[1]=str.toInt(&ok,16);
      buf[2]=0;
      str=g_codec_gbk->toUnicode(buf);
      s.replace((unsigned int) point, 14, str); // "__GB2312__xxxx "
    }

#if 1
  QFile f2("/tmp/test2.html");
  f2.open(IO_WriteOnly);
  QDataStream ds2(&f2);
  ds2.writeRawBytes((const char *)s, s.length());
#endif

    return s;
  } else {

    s=current_codec->toUnicode(s);
    return s;
    //    return current_codec->toUnicode(s);
  }
}

QString
QBookEntry::getDump (EB_Position pos, int length, bool small)
{
  EB_Position p0;
  char buf[length];
  char buf2[80];
  char *bp;
  int width=16;
  int len;
  QString text="";
  int count=0,count2=0;

  if (small) width=8;
  
  if (eb_tell_text (&book, &p0) != EB_SUCCESS)
    p0.page = -1;
  if (eb_seek_text (&book, &pos) != EB_SUCCESS)
    return QString ("Seek Error\n");

  eb_read_rawtext(&book,length,buf,&len);
  sprintf(buf2,"%d:%d - %d bytes\n",pos.page,pos.offset,len);
  text+=buf2;

  if (len==0) return QString("Read Error\n");
  bp=buf;
  while(len>0) {
    sprintf(buf2,"%04d: ",count2);
    count2+=width;
    text+=buf2;
    while(len>=0 && count<width) {
      sprintf(buf2,"%02x",0xff & bp[count]);
      text+=buf2;
      if ((count & 1)==1) text+=" ";
      count++;
      len--;
    }
    if (count<width) {
      for(int i=count;i<width;i++){
	text+="  ";
	if ((i & 1)==1) text+=" ";
      }
    }
    text+=": ";
    for(int i=0;i<count-1;i++){ 
      if (current_codec==g_codec_euc) {
	if (bp[i]>0x20 && bp[i]<0x7f && bp[i+1]>0x20 && bp[i+1]<0x7f) {
	  sprintf(buf2,"%c%c",bp[i]|0x80, bp[i+1]|0x80);
	  text+=buf2;
	  i++;
	} else  {
	  text+="..";
	  i++;
	}
      } else {
	if ((0x7f & bp[i])>=0x20) {
	  text+=bp[i];
	} else {
	  text+=".";
	}
      }
    }
    bp+=count;
    text+="\n";
    count=0;
  }
  // return text;
  return current_codec->toUnicode(text);
}

QBookEntry::~QBookEntry ()
{
  delete [] hits;
  eb_finalize_hookset (&heading_hookset);
  eb_finalize_hookset (&text_hookset);
  eb_finalize_book (&book);
  eb_finalize_appendix (&appendix);
}

void QBookEntry::setGroupNumber(int n) {
  group = n;
}

int QBookEntry::getGroupNumber() {
  return group;
}

int QBookEntry::group;

bool QBookEntry::operator<(const QBookEntry& be) const {
  int p0=priority[group], p1=be.priority[group];
  if (p0==0) p0=9999;
  if (p1==0) p1=9999;
  return p0 < p1;
}

bool QBookEntry::operator==(const QBookEntry& be) const {
  return priority[group] == be.priority[group];
}

bool QBookEntry::getNextEntry(EB_Position& pos) {
  if (!opened) return false;
  if (eb_seek_text(&book, &pos) != EB_SUCCESS) return false;
  if (eb_forward_text(&book, &appendix) == EB_SUCCESS) {
    eb_tell_text(&book, &pos);
    return true;
  }
  return false;
}

bool QBookEntry::getPrevEntry(EB_Position& pos) {
  if (!opened) return false;
  if (eb_seek_text(&book, &pos) != EB_SUCCESS) return false;
  if (eb_backward_text(&book, &appendix) == EB_SUCCESS) {
    eb_tell_text(&book, &pos);
    return true;
  }
  return false;
}

bool QBookEntry::shortCutInitialized=false;
//QPixmap* QBookEntry::shortCutPixmaps[36]; // sizeof(LinkKeys)/sizeof(LinkKeyStruct)];
//QPixmap* QBookEntry::shortCutPixmaps2[2];
QFont QBookEntry::font;

  /*
void QBookEntry::setupShortCutPixmaps() {
  int size=font.pixelSize();
  for(unsigned int i=0; i < sizeof(LinkKeys)/sizeof(LinkKeyStruct); i++) {
    if (shortCutInitialized) delete shortCutPixmaps[i];
    QPixmap* px=shortCutPixmaps[i]=new QPixmap(size, size,-1);
    QPainter p;
    px->fill(QColor(0xff,0,0));
    p.begin(px);
    
    p.setPen(QPen(QColor(0xff,0xff,0xff),0));

    p.drawPoint(0,0);
    p.drawPoint(1,0);
    p.drawPoint(0,1);

    p.drawPoint(size-1,0);
    p.drawPoint(size-2,0);
    p.drawPoint(size-1,1);

    p.drawPoint(0,size-1);
    p.drawPoint(1,size-1);
    p.drawPoint(0,size-2);

    p.drawPoint(size-1,size-1);
    p.drawPoint(size-2,size-1);
    p.drawPoint(size-1,size-2);

    p.setFont(font);
    QFontMetrics fm = p.fontMetrics();
    QString s;
    s.sprintf("%c",LinkKeys[i].ch);
    p.drawText(size/4,fm.ascent(),s);
    p.end();
  }

  if (shortCutInitialized) delete shortCutPixmaps2[0];
  QPixmap *px=shortCutPixmaps2[0]=new QPixmap(size/2,size,-1);
  QPainter p;
  px->fill(QColor(0xff,0xff,0xff));
  p.begin(px);
  p.setPen(QPen(QColor(0xff,0x0,0x0),0));
  int s2=size/2;
  for(int i=0;i<s2;i++){
    p.drawLine(i,i,i,size-i-1);
  }
  p.end();

  if (shortCutInitialized) delete shortCutPixmaps2[1];
  px=shortCutPixmaps2[1]=new QPixmap(size/2,size,-1);
  px->fill(QColor(0xff,0xff,0xff));
  p.begin(px);
  p.setPen(QPen(QColor(0xff,0x0,0x0),0));
  for(int i=0;i<s2;i++){
    p.drawLine(i,s2-i-1,i,s2+i);
  }
  p.end();

  shortCutInitialized=true;
}
  */


bool QBookEntry::menuPosition(EB_Position* pos){
  if (eb_have_menu(&book)) {
    if (eb_menu(&book, pos) == EB_SUCCESS) {
      return true;
    }
  }
  pos->page=-1;
  return false;
}

static bool indexPosition(EB_Book* book, EB_Position* pos, int indexID) {
  char buf[2048];
  int len;
  EB_Position p;
  pos->page=-1;
  p.page=1;
  p.offset=0;
  if (eb_seek_text(book, &p) == EB_SUCCESS &&
      eb_read_rawtext(book,EB_SIZE_PAGE,buf,&len) == EB_SUCCESS &&
      len==EB_SIZE_PAGE) {
    int nIndex=buf[1];
    for(int i=0;i<nIndex;i++){
      if (buf[i*16+16]!=indexID) continue;
      pos->page=getDWord(buf+i*16+16+2);
      pos->offset=0;
      return true;
    }
  }
  
  return false;
}

bool QBookEntry::imageMenuPosition(EB_Position* pos){
  EB_Disc_Code disc_code;
  if (eb_disc_type(&book, &disc_code) != EB_SUCCESS ||
      disc_code != EB_DISC_EPWING) return false;
  return indexPosition(&book, pos, 0x10);
}

bool QBookEntry::crossSearchPosition(EB_Position* pos){
  return indexPosition(&book, pos, 0x81);
}
