#ifndef QBOOKENTRY_H
#define QBOOKENTRY_H

#include <qobject.h>
#include <qlist.h>
#include <qarray.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qwidget.h>
#include <qfont.h>
#include <eb/eb.h>
#include <eb/text.h>
#include <eb/font.h>
#include <eb/error.h>
#include <eb/appendix.h>
//#include "ztentext.h"
#include "ztenconf.h"

/*
#define MAX_HITS_BASE	   100
#define MAX_HITS_SIZE	   3
#define MAX_HITS_THRESHOLD 3
*/
#define MAX_HITS	   100
#define MAX_HEADING	   100
#define MAX_TEXT	   1000

#define MAX_LENGTH 10000
#define MAX_BINARY 1000000

#define AUTO2_MATCH   -2
#define AUTO_MATCH    -1
#define FORWARD_MATCH  0
#define BACKWARD_MATCH 1
#define EXACT_MATCH    2
#define KEYWORD_MATCH  3
#define CROSSWORD_MATCH  4

extern QTextCodec* g_codec_euc;
extern QTextCodec* g_codec_gbk;
extern QTextCodec* g_codec_latin1;

enum {IMG_NA, IMG_JPEG, IMG_BMP, IMG_MONO};
enum {LINK_SOUND = -1, LINK_MOVIE = -2, LINK_IMAGE = -3, 
      LINK_IMAGE_GROUP = -4, /*LINK_XASOUND = -5,*/ LINK_ERROR = 0};
enum {MM_IMAGE, MM_SOUND,MM_MOVIE,MM_IMAGE_GROUP/*, MM_XASOUND*/};
enum {MM_SOUND_PCM};

class QBookBrowser;

struct MMItem {
  int type; // IMG, SOUND, ...
  int subType; // JPEG, BMP, MONO...
  EB_Position begin, end;
  int width, height;
  int args[4];
};


class QBookEntry : public QObject
{
  Q_OBJECT
 public:
  QBookEntry (QString title, QString path, int sub_book_pos);
  ~QBookEntry ();

  void setAppendix(QString appendix_path) {appendixPath = appendix_path;};
  void setAltTitle(QString alt_title, QString alt_title_color);
  void setKanaConv(bool kana);

  bool bindBook();
  void unbindBook();
  static int back_color, fore_color;
  
  // Return hit number.
  int search (QString keyword, int match_type = AUTO_MATCH);
  int multiSearch (EB_Multi_Search_Code code,
		   QList<QString> keywords);
  //  static void setFont (const QFont& f) { font = f; setupShortCutPixmaps(); }
  static void setFont (const QFont& f) { font = f; };
  int getFontSize () { return font.pixelSize(); }
  //  void setImageScale (int scale) { img_scale = scale; }
  //  int getImageScale () { return img_scale; }
  void findMultiSearch (bool all=false);
  void findMultiSearchEntry (int multi_id);

  QString getHeading (int hit_pos);

  bool getPrevEntry(EB_Position& pos);
  bool getNextEntry(EB_Position& pos);

  // To compare identity;
  QString getIdentity ();
  bool equals (QString identity);

  EB_Position getHitPosition (int hit_pos);
  EB_Position getLinkPosition (int link_pos);
  
  QString getText (EB_Position pos, QBookBrowser* qb);
  QString getDump (EB_Position pos, int length=2048, bool small=false);
  void addPositionString (QString *str);
  EB_Book *getBook () {return &book;}
  QString getTitle ();
  QString getAltTitle ();
  QString getAltTitleColor ();

  // Multi Media Items
  QArray<MMItem> mmarray;

  // temporary 
  int img_type;
  int img_no;
  EB_Position img_pos;
  int img_width, img_height;

  void playSound (int index);
#if 0
  void playXASound (int index);
#endif
  void playMovie (int index);
  void showImage (int index);
  bool showImageGroup (int index, EB_Position& pos);
  bool showImageGroup (EB_Position& pos);

  bool isOpened (); // Check zten level status
  bool isBound ();  // Check EB Library Status
  bool isReady ();  // Check Disk Status


  bool menuPosition(EB_Position* pos);
  bool imageMenuPosition(EB_Position* pos);
  bool crossSearchPosition(EB_Position* pos);

  // To identify subbook;
  QString absolute_path;
  int sub_book_pos;
  
  QString title;
  int memory;
  int priority[MAX_DIC_GROUPS+2];
  QString alt_title;
  QString alt_title_color;
  //  bool appendix_enabled;
  QString appendixPath;
  bool kanaConv;
  int stop_code_mode, stop_code0, stop_code1;

  bool menu_enabled;

  //  bool continuous_mode;
  bool multi_search_enabled[EB_MAX_MULTI_SEARCHES];
  QString multi_search_alt_title[EB_MAX_MULTI_SEARCHES];
  int n_multi_searches;

  QTextCodec* current_codec;

 private:
  //  int font_size;
  EB_Book book;
  EB_Subbook_Code sub_code;
  EB_Character_Code ccode;
  EB_Hookset heading_hookset;
  EB_Hookset text_hookset;
  EB_Appendix appendix;
  QList<QString> pos_list;
  
  QString last_heading; // For duplication.

  EB_Hit *hits;
  int hit_count;

  bool isRedundantEntry (int);
  void openAppendix (QString,EB_Subbook_Code);

  EB_Error_Code search_keyword(int match_type, EB_Book *book,
			       const char * const input_words[]);

  bool opened;

  static bool shortCutInitialized;
  //  static QPixmap* shortCutPixmaps[36];

signals:
  // this,multi_id,entry_number
  void foundMultiSearch (QBookEntry *,int,QString,int);
  // this,entry_index,label
  void foundMultiSearchEntry (QBookEntry *,int,QString,bool);

 public:
  static void setGroupNumber(int n);
  static int getGroupNumber();
  static int group;

  //  static void setupShortCutPixmaps();
  static QFont font;

  bool operator<(const QBookEntry& be) const;
  bool operator==(const QBookEntry& be) const;
};

#endif // QBOOKENTRY_H
