/*                                                            -*- C -*-
 * Copyright (c) 2003-2004 Teranishi & Nemoto 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef ZTEN_H
#define ZTEN_H

#include <eb/eb.h>
#include <eb/error.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <qlist.h>
#include <qsortedlist.h>
#include <qlistview.h>
#include <qlistbox.h>
//#include <qfont.h>
#include "ztenbase.h"
#include "ztenconf.h"
#include "qlistboxrichtext.h"

class QHistoryLineEdit;
class QAction;
class MyPushButton;

class ZtenConf;
class QBookEntry;
class QBookItem;

class ZtenWindow;

class ZtenBookList;
class ZtenMultiSearch;

class QFont;
class QTimer;
class QPopupMenu;
class QLineEdit;

class QProgressBar;

class QBookHitItem : public QListBoxRichText
{
 public:
  QBookHitItem (const QString &label, QBookEntry *be, int hit_pos);
  ~QBookHitItem ();
  QBookEntry *getBookEntry () { return be; }
  int getHitPos () { return hit_pos; }
 private:
  QBookEntry *be;
  int hit_pos;
};

class QBookMultiSearch
{
 public:
  QBookMultiSearch (QBookEntry *be,int multi_id)
  { this->be = be; this->multi_id = multi_id; }
  ~QBookMultiSearch () {};
  QBookEntry *getBookEntry () { return be; }
  int getMultiId () { return multi_id; }
  bool operator <(const QBookMultiSearch& qms);
  bool operator ==(const QBookMultiSearch& qms);
 private:
  QBookEntry *be;
  int multi_id;
};

class Zten : public ZtenBase
{
  Q_OBJECT
public:
  Zten (QWidget *parent = 0, const char* name = 0, WFlags fl = 0);
  ~Zten ();
  //  ZtenConf *config;

  void initializeGUI();

  void UpdateConfig();

  void setCurrentFont (const QFont& font);
  void setImageScale (int scale);
  void addBook (QBookEntry *item);
  void clearBooks ();
  void lookup (QString key);

  void updateDicList(bool sort=true);
  void updateMenuStatus();
  void dicList();

  int focus;
  void setFocus(int focusValue);
  enum {FocusLineEdit = 0, FocusEntryList = 1, 
	FocusContentBrowser = 2, FocusMaximizedContentBrowser = 3
  };

  QPopupMenu *MenuSearch, *MultiSearch;
  ZtenBookList *ztenBookList;
  QListView *DictionaryList;
  ZtenMultiSearch *zm;
  QColorGroup *cg;

  bool _disk_status_changed;
  
  // These are accessed from ZtenWindow class

 protected:
  MyPushButton* DelButton;
  QHistoryLineEdit* LineEdit;
  MyPushButton* LookupButton;
  QLineEdit *JumpEdit;

 public:
  QSortedList<QBookEntry> books;
  QSortedList<QBookMultiSearch> ms_list; // current multi search list.
  QMap<int,QBookEntry*> singleBook;
  QList<QBookEntry> menu_books;
  QList<EB_Position> menu_positions;
  ZtenWindow* MainWindow;

 private:


  int enabled;
  bool displaying;
  int current_content_mode;

  bool entryListSelected;

  bool waitCursorMode;
  QTimer *idleTimer;
  void setWaitCursor();
  void resetWaitCursor();

  // FIXME: QBookBrowser should have these member.
  QBookEntry *current_book;
  QArray<EB_Position> history;
  QArray<QBookEntry*> history_book;
  int history_current;

  void goToNoHistory (EB_Position pos);
  void goToHistory (int current_history);
  void addHistory(const EB_Position& pos, QBookEntry* be=NULL);
  //  bool redisplay (QBookEntry *item = NULL);
  //  bool opened;
  void linkJump (int pos);

  void showBlank();
  void showCopyright();

  void addMenuTitle(QBookEntry* be, int num, bool isImage);

public slots:
  void selectFont ();
  void setExtension (QBookEntry *be);
  void setPreferences();
  //  void deleteBook (QBookEntry *be);
  void cut (); // Assigned as copy+paste
  void copy ();
  void paste ();
  void back ();
  void forward ();
  void up ();
  void down ();
  void entryListEscape ();
  void entryListUp ();
  void entryListDown ();
  //  void entryListDown2 ();
  void contentLineUp ();
  void contentLineDown ();
  void contentPageUp ();
  void contentPageDown ();
  //  void toggleContent ();
  void toggleDirection ();
  void setIncremental (bool);
  void setAutoContent (bool);
  void setClipSearch (bool);
  void closeBooks ();
  void diskChanged();
  void reload ();

  void showInfo();
  void dumpCurrent();
  
  void linkJump();

  //  void searchClipboard ();

  void addMultiSearch (QBookEntry*,int,QString,int);
  void addMultiSearchEntry (QBookEntry*,int,QString,bool);

  //  int openBooks (bool all=false,QProgressBar* progress=NULL);
  int openBooks (bool,QProgressBar* progress);
  void closeSD();
  void closeCF();
  void findMultiSearches (bool all=false);
  void findMenues (bool all=false);

  void search ();

 protected:
  void resizeEvent (QResizeEvent* e);
  bool eventFilter (QObject *o, QEvent *e);
  bool eventFilterJump (QObject *o, QEvent *e);
  bool eventFilterMaximizedContent (QObject *o, QEvent *e);
  bool eventFilterNormal (QObject *o, QEvent *e);
  bool eventFilterCommon (QObject *o, QEvent *e);
  bool waitUnlock(int num);


  enum content_mode { MaximizeEntryList=1, MaximizeContent=2, DisplayBoth=3,
		      AddEntryList=4, AddContent=5, 
		      LargerEntryList=6, LargerContent=7,
		      ToggleEntryList=8,
  };

 private slots:
  void contentMode (content_mode m, bool redrawContent = true);
  void clearLine ();
  //  void closeBooks ();
  //  void entryListUp();
  //  void entryListDown();
  void activateAndSearch ();
  void searchIncremental (const QString &word);
  void goTo (EB_Position);
  void itemSelected (QListBoxItem *item = NULL);
  void itemPressed (QListBoxItem *item = NULL);
  void playSound (int);
  void playMovie (int);
  void showImage (int);
  void showImageGroup (int);
  void menuResume ();
  
  void gotoMenu (QBookEntry *, EB_Position *);
  void menuSelected (int);
  void multiSelected (int);
  void execSearch();
  void execSearch2();
  void idle_in();

//   int changeDirectory(const char*);
signals:
  // history signal
  void noHistory (); // disable back, forward
  void beginningOfHistory (); // disable back
  void endOfHistory (); // disable forward
  void makeHistory (); // enable back, disable forward
  void followHistory (); // enable back, forward
  // tab signal
  void contentSelected ();
  void booklistSelected ();
  //  void incrementalIsSet (bool);

  void reloadBooks ();

 public:
  friend class ZtenMultiSearch;
};

#endif // ZTEN_H
