/*                                                            -*- C -*-
 * Copyright (c) 2004 Takashi Nemoto 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "ztenbooklist.h"
#include "mypushbutton.h"
#include "1downarrow.xpm"
#include "1uparrow.xpm"
#include "edittrash.xpm"
#include "zten.h"
#include "qbookentry.h"
#include "qbookitem.h"
#include "diskstat.h"
#include "zpixmap.h"
#include "ztenconf.h"
#include "utils.h"

#ifndef ZAURUS
#include <qhbox.h>
#include <qlayout.h>
#endif
#include <qevent.h>
#include <qtextcodec.h>
#include <qprogressbar.h>

#include <dirent.h>

extern QTextCodec* g_codec_euc;
extern QTextCodec* g_codec_gbk;
extern QTextCodec* g_codec_latin1;

class MyProgressBar : public QProgressBar {
  bool update;
 public:
  MyProgressBar(const QString& t, int totalSteps, QWidget * parent=0, 
		const char * name=0, WFlags f=0 ) : 
    QProgressBar (totalSteps, parent, name, f) {
    title=t;
    update=true;
  }
  QString title;
 protected:
  virtual bool setIndicator( QString & indicator, int /*progress*/, 
			     int /*totalSteps*/ ) {
    /*
    indicator=title+" "+QString::number(progress)+"/"+
      QString::number(totalSteps);
    return true;
    */
    indicator=title;
    if (update) {
      update=false;
      return true;
    } else {
      return false;
    }
  };
};

ZtenBookList::ZtenBookList(Zten *zt, QWidget* parent, const char* name, bool modal, WFlags fl) :   ZtenBookListBase(parent, name, modal, fl) {
#ifndef ZAURUS
  QHBox* hbox=new QHBox(this);;
  QPushButton *acceptButton=new QPushButton(tr("Accept"),hbox);
  QPushButton *rejectButton=new QPushButton(tr("Reject"),hbox);
  hbox->setMaximumSize( QSize( 32000, 30 ) );
  connect((QObject *)acceptButton, SIGNAL (clicked()), this, SLOT (accept ()));
  connect((QObject *)rejectButton, SIGNAL (clicked()), this, SLOT (reject ()));
  ZtenBookListBaseLayout->addWidget(hbox);
#endif  

  zten = zt;

  // ==== parent->setToolBarsMovable(FALSE);
  setCaption (tr("Zten Variant Dictionary Configuration"));


  DicEditButton->setText(tr("Find Dictionaries"));
  DicConfigButton->setText(tr("Dic Property"));
  DicUpButton->setPixmap(ZPIXMAP (uparrow_xpm));
  DicDownButton->setPixmap(ZPIXMAP (downarrow_xpm));
  DicTrashButton->setPixmap(ZPIXMAP (edittrash_xpm));

  connect (DicEditButton, SIGNAL (clicked ()), this,
	   SLOT (selectBooks ()));
  connect (DicDownButton, SIGNAL (clicked ()), this, SLOT (listDown ()));
  connect (DicUpButton, SIGNAL (clicked ()), this, SLOT (listUp ()));

  connect (DicConfigButton, SIGNAL (clicked ()), this,
	   SLOT (setExtension ()));
  connect (DicTrashButton, SIGNAL (clicked ()), this,
	   SLOT (deleteBook ()));

  connect(CoreButton, SIGNAL (clicked()), this, SLOT (setPriority0()));
  connect(Renumber1Button, SIGNAL (clicked()), this, SLOT (setPriority1()));
  connect(Renumber2Button, SIGNAL (clicked()), this, SLOT (setPriority2()));
  connect(Renumber3Button, SIGNAL (clicked()), this, SLOT (setPriority3()));
  connect(Renumber4Button, SIGNAL (clicked()), this, SLOT (setPriority4()));
  connect(Renumber5Button, SIGNAL (clicked()), this, SLOT (setPriority5()));
  connect(Renumber6Button, SIGNAL (clicked()), this, SLOT (setPriority6()));
  connect(Renumber7Button, SIGNAL (clicked()), this, SLOT (setPriority7()));
  connect(Renumber8Button, SIGNAL (clicked()), this, SLOT (setPriority8()));
  connect(Renumber9Button, SIGNAL (clicked()), this, SLOT (setPriority9()));
  connect(RenumberSingleButton, SIGNAL (clicked()), this, SLOT (setPriorityS()));
  connect(RenumberAllButton, SIGNAL (clicked()), this, SLOT (setPriorityA()));

  CoreButton->setText("&C");
  Renumber1Button->setText("&1");
  Renumber2Button->setText("&2");
  Renumber3Button->setText("&3");
  Renumber4Button->setText("&4");
  Renumber5Button->setText("&5");
  Renumber6Button->setText("&6");
  Renumber7Button->setText("&7");
  Renumber8Button->setText("&8");
  Renumber9Button->setText("&9");
  RenumberSingleButton->setText("&S");
  RenumberAllButton->setText("&A");
  installEventFilter(this);
  //  zten->DictionaryList
};

bool
ZtenBookList::eventFilter(QObject *o, QEvent *e) {
  QKeyEvent *k = (QKeyEvent *) e;
  bool r=ZtenBookListBase::eventFilter(o,e);
  keyState=k->state();
  return r;
}

ZtenBookList::~ZtenBookList() {};


void ZtenBookList::setPriority0() {
  setPriority(0);
}
void ZtenBookList::setPriority1() {
  setPriority(1);
}
void ZtenBookList::setPriority2() {
  setPriority(2);
}
void ZtenBookList::setPriority3() {
  setPriority(3);
}
void ZtenBookList::setPriority4() {
  setPriority(4);
}
void ZtenBookList::setPriority5() {
  setPriority(5);
}
void ZtenBookList::setPriority6() {
  setPriority(6);
}
void ZtenBookList::setPriority7() {
  setPriority(7);
}
void ZtenBookList::setPriority8() {
  setPriority(8);
}
void ZtenBookList::setPriority9() {
  setPriority(9);
}
void ZtenBookList::setPriorityS() {
  setPriority(10);
}

void ZtenBookList::setPriorityA() {
  setPriority(11);
}

void ZtenBookList::setPriority(int n) {
  int p=1;
  QString s;
  for(QListViewItemIterator it(DictionaryList); it.current(); ++it) {
    QBookItem *bi =(QBookItem*) it.current();
    for(QBookEntry *be=zten->books.first(); be; be=zten->books.next()) {
      if (be->absolute_path == bi->getPath() && 
	  be->sub_book_pos ==bi->getSubBookPos()) {
	int x=be->priority[n];
	if (bi->isSelected()) {
	  if (x>0 && (keyState & ControlButton)!=0) {
	    x=0;
	  } else {
	    x=p++;
	  }
	} else {
	  if (x>0 && (keyState & (ShiftButton|ControlButton))!=0) {
	    x=p++;
	  } else {
	    x=0;
	  }
	}
	be->priority[n]=x;
	s.sprintf("%03d",x);
	if (x==0) s="ZZZ";
	bi->setText(2+n,s);
        bi->setColor(be->alt_title_color);
      }
    }
  }
  DictionaryList->clearSelection();
}

void
ZtenBookList::listDown ()
{
  QListViewItemIterator it(DictionaryList),it2;
  for(; it.current(); ++it) it2=it;
  for(it=it2; it.current(); --it) {
    QListViewItem *bi = it.current();
    if (bi->isSelected() && bi->itemBelow () && !bi->itemBelow()->isSelected())
      {
	bi->moveItem (bi->itemBelow ());
      }
  }
  DictionaryList->ensureItemVisible (DictionaryList->selectedItem ());  
}

void
ZtenBookList::listUp ()
{
  for(QListViewItemIterator it(DictionaryList); it.current(); ++it) {
    QListViewItem *bi = it.current();
    if (bi->isSelected() && 
	bi->itemAbove () && !bi->itemAbove()->isSelected()) {
      bi->itemAbove()->moveItem (bi);
    }
  }
  DictionaryList->ensureItemVisible (DictionaryList->selectedItem ());  
}

void
ZtenBookList::setExtension ()
{
  for(QListViewItemIterator it(DictionaryList); it.current(); ++it) {
    QBookItem *bi = (QBookItem *) it.current();
    if (bi->isSelected()) {
      for(QBookEntry *be=zten->books.first(); zten->books.current();
	  be=zten->books.next()) {
	if (be->absolute_path == bi->getPath() && 
	    be->sub_book_pos ==bi->getSubBookPos()) {
	  zten->setExtension (be);
	  break;
	}
      }
    }
  }
  zten->updateDicList(false);
}

void
ZtenBookList::deleteBook ()
{ 
  bool flag=true;
  do {
    flag=false;
    for(QListViewItemIterator it(DictionaryList); it.current(); ++it) {
      QBookItem *bi = (QBookItem *) it.current(); if (bi->isSelected()) {
	for(QBookEntry *be=zten->books.first(); be; be=zten->books.next()) {
	  if (be->absolute_path == bi->getPath() && 
	      be->sub_book_pos ==bi->getSubBookPos()) {
	    DictionaryList->removeItem (bi);
	    zten->books.remove();
	    flag=true;
	    break;
	  }
	}
      }
    }
  } while (flag);
}

void ZtenBookList::selectBooks(void) {
  const char *curd;
  int i = 0; 

#ifdef ZAURUS
  bool existZaurusUser=false;
#endif

  QProgressBar *progress = NULL;
  int dircount = 0;

  for (curd = g_book_path[i].path; curd != NULL; curd = g_book_path[++i].path){
    struct dirent **dir;
    int k, j, n_files;
    EB_Book book;
    EB_Subbook_Code subbook_list[EB_MAX_SUBBOOKS];
    int subbook_count;
    char title[EB_MAX_TITLE_LENGTH + 1];
    EB_Character_Code ccode;
    QTextCodec* current_codec;

    if (chdir(curd) < 0) continue;
    if ((n_files = scandir(".", &dir, 0, alphasort)) < 0) {
      free(dir);
      continue;
    }

#ifdef ZAURUS
    // Skip /home/root/Documents/*  if exist /home/zaurus/Documents/* 
    if (i==1 && existZaurusUser) continue;
    existZaurusUser=true;
#endif

    if (config->EnableProgressBar) {
      progress=new MyProgressBar(g_book_path[i].name,n_files,this);
      progress->move(0,40+(dircount++)*20);
      progress->resize(200,20);
      progress->show();
    }

    for(k = 0; k < n_files; k++) {
      if (config->EnableProgressBar) {
	progress->setProgress(k);
      }
      if (strcmp(dir[k]->d_name, ".") == 0 || 
	  strcmp(dir[k]->d_name, "..") == 0) 
	continue;

      eb_initialize_book (&book);
      if (eb_bind (&book, dir[k]->d_name) != EB_SUCCESS) {
	eb_finalize_book (&book);
	free (dir[k]);
	continue;
      }
      current_codec=g_codec_euc;
      if (eb_character_code(&book,&ccode) == EB_SUCCESS) {
	if (ccode == EB_CHARCODE_ISO8859_1) {
	  current_codec=g_codec_latin1;
	}
      }

      if (eb_subbook_list(&book, subbook_list, &subbook_count) != EB_SUCCESS)
	continue;

      for (j = 0; j < subbook_count; j++) {
	if (eb_subbook_title2 (&book, subbook_list[j], title) != EB_SUCCESS)
	  continue;
	if (ccode!=EB_CHARCODE_ISO8859_1)
	  Utils::z2h(title);
	QBookEntry *be;
	QString path=QString(curd)+QString(dir[k]->d_name);
	for(be=zten->books.first(); be != NULL; be = zten->books.next()) {
	  if (be->absolute_path == path && 
	      be->sub_book_pos == j) break;
	}
	if (be != NULL) continue; // already in list
	zten->addBook (new QBookEntry(current_codec->toUnicode(title),path,j));

      }
      eb_finalize_book(&book);
      free(dir[k]);
    }
    if (config->EnableProgressBar) {
      progress->setProgress(n_files);
      delete progress;
    }
    free(dir);
  }

  chdir("/tmp");

  zten->updateDicList();


  if (config->EnableProgressBar) {
    progress=new QProgressBar(zten->books.count(),this);
  } else 
    progress=NULL;

  zten->openBooks(true,progress);
  zten->findMenues(true);
  zten->findMultiSearches(true);
}


void ZtenBookList::accept() {
  ZtenBookListBase::accept();
}

