/*                                                            -*- C -*-
 * Copyright (c) 2003-2004 Teranishi & Nemoto 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <qdialog.h>
#include <qtextview.h>
#include <qlayout.h>
// #include <qvariant.h>
// #include <qtooltip.h>
// #include <qwhatsthis.h>

#include "mypushbutton.h"
#include <qlabel.h>
#include <qlineedit.h>
#include <qhbox.h>
#include <qvbox.h>

#include "zten.h"
#include "ztendump.h"
#include "qbookentry.h"

ZtenDump::ZtenDump (QWidget* parent,
		    const char* name, bool modal, WFlags fl) : 
  QDialog(parent, name, modal, fl)
{
  //    QVBox* ZtenDumpBaseLayout;
    QVBoxLayout* ZtenDumpBaseLayout;
    if ( !name )
	setName( "ZtenDumpBase" );
    resize( 592, 400 ); 
    setCaption( tr( "Zten Information" ) );
    ZtenDumpBaseLayout = new QVBoxLayout( this ); 
    ZtenDumpBaseLayout->setSpacing( 2 );
    ZtenDumpBaseLayout->setMargin( 2 );
    //    ZtenDumpBaseLayout->setMaximumSize( QSize( 32767, 32767 ) );
    //    ZtenDumpBaseLayout->setMinimumSize( QSize( 200,200 ));

    AddressBar = new QHBox(this);


    //    QLabel *PageLabel = new QLabel( tr("Page:"), AddressBar );
    DumpPageEntry = new QLineEdit( AddressBar, "DumpPageEntry" );
    DumpPageEntry->setMinimumSize( QSize( 50, 18 ) );

    //    QLabel* OffsetLabel = new QLabel( tr("Offset"), AddressBar);
    /* QLabel* OffsetLabel = */ new QLabel( ":", AddressBar);
    DumpOffsetEntry = new QLineEdit( AddressBar, "DumpPageEntry" );
    DumpOffsetEntry->setMinimumSize( QSize( 50, 18 ) );

    DumpExecButton = new MyPushButton( AddressBar, "Jump" );
    DumpExecButton->setText(tr("Jump"));

    ZtenDumpBaseLayout->addWidget( AddressBar );

    TextView = new QTextView( this, "TextView" );

    ZtenDumpBaseLayout->addWidget( TextView );

    //    ZtenDumpBaseLayout->setStretchFactor(PathLabel,1);
    //    ZtenDumpBaseLayout->setStretchFactor(AddressBar,1);
    //    this->setStretchFactor(ZtenDumpBaseLayout,1);
    ZtenDumpBaseLayout->setStretchFactor(TextView,1);

  codec = QTextCodec::codecForName("eucJP");

  connect ((QObject *)DumpExecButton, SIGNAL (clicked()), this, SLOT (go ()));
  connect ((QObject *)DumpPageEntry, SIGNAL (returnPressed()), this, SLOT (go ()));
  connect ((QObject *)DumpOffsetEntry, SIGNAL (returnPressed()), this, SLOT (go ()));

#ifndef ZAURUS
  QHBox* hbox=new QHBox(this);;
  QPushButton *acceptButton=new QPushButton(tr("Accept"),hbox);
  QPushButton *rejectButton=new QPushButton(tr("Reject"),hbox);
  hbox->setMaximumSize( QSize( 32000, 30 ) );
  connect((QObject *)acceptButton, SIGNAL (clicked()), this, SLOT (accept ()));
  connect((QObject *)rejectButton, SIGNAL (clicked()), this, SLOT (reject ()));
  ZtenDumpBaseLayout->addWidget(hbox);
#endif  
  TextView->setFont(QFont(config->FontFamily, config->FontSize));
}

ZtenDump::~ZtenDump ()
{
}

void
ZtenDump::go ()
{
  setAddress(cbe, DumpPageEntry->text().toInt(),
	     DumpOffsetEntry->text().toInt());
}

void ZtenDump::setAddress(QBookEntry *be, int page, int offset) {
  cbe=be;
  QString path;
  EB_Position p;
  p.page=page;
  p.offset=offset;
  if (offset<0 || offset>=2048) {
    setCaption(tr("Dump"));
    setText(tr("Can not dump current page"));
    return;
  } else {
    QString subbook;
    subbook.setNum(cbe->sub_book_pos);
    path=cbe->absolute_path+":"+subbook;
    setText(cbe->getDump(p,2048,false));
  }
    

  setCaption( path);
  if (offset>=0) {
    QString s;
    s.setNum(page);
    DumpPageEntry->setText(s);
    s.setNum(offset);
    DumpOffsetEntry->setText(s);
  } else {
    /*
    DumpPageEntry->setText("------");
    DumpPageEntry->setReadOnly(true);
    DumpOffsetEntry->setText("----");
    DumpOffsetEntry->setReadOnly(true);
    DumpExecButton->disable();
    */
    AddressBar->hide();
  }
}

void ZtenDump::setText(const QString& str) {
  TextView->setText(str);
}
