/*                                                            -*- C -*-
 * Copyright (c) 2003-2004 Teranishi & Nemoto 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <qdialog.h>
#include <qlayout.h>

#include <qlabel.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qscrollview.h>
#include <qpixmap.h>

#include "ztenimage.h"

ZtenImage::ZtenImage (QWidget* parent,
		      const char* name, bool modal, WFlags fl) : 
  QDialog(parent, name, modal, fl)
{

  setResult(0);
  QVBoxLayout* ZtenImageBaseLayout;
  if ( !name )
    setName( "ZtenImageBase" );
  resize( 640, 480 ); 
  setCaption( tr( "Zten Image Viewer" ) );
  ZtenImageBaseLayout = new QVBoxLayout( this ); 
  ZtenImageBaseLayout->setSpacing( 2 );
  ZtenImageBaseLayout->setMargin( 2 );

  QScrollView *view = new QScrollView(this);
  ZtenImageBaseLayout->addWidget( view );
  QVBox *vbox = new QVBox(view->viewport());
  view->addChild(vbox);

  image = new QLabel(vbox);
  caption = new QLabel(vbox);
}

ZtenImage::~ZtenImage ()
{
}

void ZtenImage::setImage(const QImage& qi, const QString& str) {
  QPixmap pix;
  pix.convertFromImage(qi);
  image->setPixmap(pix);
  caption->setText(str);
  image->installEventFilter (this);
}

void 
ZtenImage::addArea(int x0,int y0,int x1, int y1, int page, int offset) {
  int count=area.count();
  area.resize(count+1);
  area[count]=ClickRect(x0,y0,x1,y1,page,offset);
}

bool
ZtenImage::eventFilter (QObject* /* o */, QEvent* e)
{
  if (e->type() == QEvent::MouseButtonRelease) {
    QMouseEvent *m = (QMouseEvent *) e;
    if (findArea(m->x(),m->y(),pos)) {
      done(1);
    }
    return TRUE;
  } else if (e->type () == QEvent::KeyPress) {
    QKeyEvent *k = (QKeyEvent *)e;
    int key=k->key();
    int scaling=5;
    if (k->state() & Qt::ShiftButton) scaling=2;
    switch (key) {
    case Qt::Key_Up:
      view->scrollBy (0, -view->height() / scaling);
      return TRUE;
    case Qt::Key_Down:
      view->scrollBy (0, view->height() / scaling);
      return TRUE;
    case Qt::Key_Left:
      view->scrollBy (-view->width() / scaling, 0);
      return TRUE;
    case Qt::Key_Right:
      view->scrollBy (view->height() / scaling, 0);
      return TRUE;
    }
    return FALSE;
  }
  return FALSE;
}

ClickRect::ClickRect(int X0, int Y0, int X1, int Y1, int page, int offset) :
  x0(X0), y0(Y0), x1(X1), y1(Y1) {
  addr.page=page;
  addr.offset=offset;
}
  
bool ZtenImage::findArea(int x, int y, EB_Position& pos) {
  int count=area.count();
  for(int i=0;i<count;i++) {
    ClickRect& a=area[i];
    if (a.x0<=x && x<a.x1 && a.y0<=y && y<a.y1) {
      pos.page=a.addr.page;
      pos.offset=a.addr.offset;
      return true;
    }
  }
  return false;
}

void ZtenImage::accept(void) {
  if (area.count()==1) {
    pos=area[0].addr;
  } else {
    pos.page=-1;
  }
  QDialog::accept();
}
