/*                                                            -*- C -*-
 * Copyright (c) 2003-2004 Teranishi & Nemoto 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <qiconset.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qcursor.h>
#ifdef ZAURUS
#include <qpe/qpetoolbar.h>
#include <qpe/qpemenubar.h>
#include <qwindowsystem_qws.h>
#include <qgfx_qws.h>
#include <qcopchannel_qws.h>
#else
#include <qtoolbar.h>
#include <qmenubar.h>
#define QPEMenuBar QMenuBar
#endif
#include <qaction.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <qfont.h>
#include <qfontdatabase.h>
#include <qapplication.h>
#include <qclipboard.h>
#include "mypushbutton.h"
#include "ztenwindow.h"
#include "zpixmap.h"
#include "diskstat.h"
#include "qhistorylineedit.h"
#include <qlineedit.h>
#include <qlabel.h>
#include "ztenconf.h"
#include "qbookbrowser.h"
#include <qtooltip.h>
#include "qbookentry.h"
#include <qtimer.h>

#include <time.h>

#include "editcopy.xpm"
#include "editpaste.xpm"
#include "back.xpm"
#include "forward.xpm"
#include "down.xpm"

#include "delbutton.xpm"

#ifdef ZAURUS
#include <qpe/qpeapplication.h>
extern QPEApplication *app;
#include <qpe/qcopenvelope_qws.h>
#else
#include <qapplication.h>
extern QApplication *app;
#endif

#ifdef ZAURUS
#if 0
extern "C" {
  extern int zau_use_itimer;
  extern int zau_c3000_mode;
  extern int zau_opened;
  int zau_temporary_close_all(void); 
  void zau_get_zio_status(char *buf, int fd);
  extern void (*zau_open_trig) (void);
};
#endif
#endif

#ifndef ZAURUS
bool zau_opened = true;
#endif

#define NUM_SCALES 7
int scales [] = {10, 25, 50, 75, 100, 150, 200};

static int match_id[]={FORWARD_MATCH, BACKWARD_MATCH, 
		       EXACT_MATCH, KEYWORD_MATCH, CROSSWORD_MATCH};

//static ZtenWindow *zw;

QTextCodec* g_codec_euc;
QTextCodec* g_codec_gbk;
QTextCodec* g_codec_latin1;


#if 0
static void openTrigX(void) {
  zw->openTrig();
}
#endif

ZtenWindow::ZtenWindow (QWidget *parent) : QMainWindow(parent)
{
//  zw=this;
  setCaption ("Zten Variant");
  setToolBarsMovable(FALSE);

  g_codec_euc = QTextCodec::codecForName("eucJP");
  g_codec_latin1 = QTextCodec::codecForName("iso8859-1");
  g_codec_gbk = QTextCodec::codecForName("gb2312");

  menuBarBase = new QToolBar (this);
#ifdef ZAURUS
  menuBar = new QPEMenuBar(menuBarBase);
#else
  menuBar = new QMenuBar(this);
#endif
  menuBar->setMargin (0);
  toolBar = new QToolBar (this);
  editBar = new QToolBar(this);
  browseBar = new QToolBar(this);

  {
    searchBar = new QToolBar (this);
    DelButton= new MyPushButton(searchBar,NULL);
    DelButton->setPixmap(ZPIXMAP(delbutton_xpm));
    DelButton->setFocusPolicy(QWidget::NoFocus);
    LineEdit = new QHistoryLineEdit( searchBar, "LineEdit" );
    LineEdit->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, 
					  (QSizePolicy::SizeType)0, 
		       LineEdit->sizePolicy().hasHeightForWidth() ) );
    LineEdit->setMinimumSize( QSize( 100, 18 ) );
    LineEdit->setFocusPolicy( QHistoryLineEdit::StrongFocus );
    LineEdit->setAutoCompletion( false );
    searchBar->setStretchableWidget(LineEdit);
    /*
    LookupButton = new MyPushButton (searchBar, "LookupButton");
    LookupButton->setText( tr( "Search" ) );
    LookupButton->setFocusPolicy(QWidget::NoFocus);
    */
    new QLabel(tr("Jump"),searchBar);
    JumpEdit=new QLineEdit(searchBar,"JumpEdit");
    JumpEdit->setMaximumSize(QSize(32,18));
  }


//  editBox = new QActionGroup(this);
//  browseBox = new QActionGroup(this);
  //  searchBox = new QActionGroup(this);

#ifdef ZAURUS
#if 0
  timer2 = new QTimer();
  connect(timer2, SIGNAL(timeout ()), SLOT(closeFiles ()));
  zau_open_trig=openTrigX;

  clipSearchCount=0;
  //  lastClipboard=QApplication::clipboard()->text();
  lastClipboard="";
  timer3 = new QTimer();
  connect(timer3, SIGNAL(timeout ()), SLOT(checkClipboard ()));
  timer3 -> start(0,true);
#endif
#endif

  copyAction = new QAction (tr ("Copy"),
			    ZPIXMAP (editcopy_xpm),
			    QString::null, 0, /*editBox*/NULL, 0);
  copyAction->addTo(editBar);
  connect (copyAction, SIGNAL (activated ()), this, SLOT (copy ()));

  pasteAction = new QAction (tr ("Paste"),
			     ZPIXMAP (editpaste_xpm),
			     QString::null, 0, /*editBox*/NULL, 0);
  pasteAction->addTo(editBar);
  connect (pasteAction, SIGNAL (activated ()), this, SLOT (paste ()));

  backAction = new QAction (tr ("Back"),
			    ZPIXMAP (back_xpm),
			    QString::null, 0, /*browseBox*/NULL, 0);
  backAction->addTo(browseBar);
  connect (backAction, SIGNAL (activated ()), this, SLOT (back ()));

  forwardAction = new QAction (tr ("Forward"),
			       ZPIXMAP (forward_xpm),
			       QString::null, 0, /*browseBox*/NULL, 0);
  forwardAction->addTo(browseBar);
  connect (forwardAction, SIGNAL (activated ()), this, SLOT (forward ()));
  
  downAction = new QAction (tr ("Next"),
			    ZPIXMAP (down_xpm),
			    QString::null, 0, /*browseBox*/NULL, 0);
  downAction->addTo(browseBar);
  connect (downAction, SIGNAL (activated ()), this, SLOT (down ()));

			    
  menu = new QPopupMenu (menuBar);
  menu->setCheckable(true);

//  editBox->addTo(toolBar);
//  browseBox->addTo(toolBar);
  //  searchBox->addTo(toolBar);

  //  menuDictionaryGroup = new QPopupMenu(this);
  menuDictionaryGroup = new QPopupMenu(menuBar);
  menuDictionaryGroup->setCheckable(true);
  
  match = new QPopupMenu(this);
  match->setCheckable(true);

  scale = new QPopupMenu(this);
  scale->setCheckable(true);

  menuConfigToolBar = new QPopupMenu(this);
  menuConfigToolBar->setCheckable(true);
  //=======================
  editTool_id = menuConfigToolBar->insertItem("Edit Tool", this, SLOT(toggleEditTool()));
  browseTool_id = menuConfigToolBar->insertItem("Browse Tool", this, SLOT(toggleBrowseTool()));
  searchTool_id = menuConfigToolBar->insertItem("Search Tool", this, SLOT(toggleSearchTool()));
//  titleBar_id = menuConfigToolBar->insertItem("Title Bar", this, SLOT(toggleTitleBar()));

  //  font = makeFontMenu();

  for (int i = 0; i < NUM_SCALES; i++)
    {
      QString s;
      s.sprintf ("%d %%", scales[i]);
      scale->insertItem (s, this, SLOT (setImageScale (int)));
    }
  match->insertItem(tr("Forward Match"), this, SLOT(setCurrentMatch (int)));
  match->insertItem(tr("Backward Match"), this, SLOT(setCurrentMatch (int)));
  match->insertItem(tr("Exact Match"), this, SLOT(setCurrentMatch (int)));
  match->insertItem(tr("Keyword Match"), this, SLOT(setCurrentMatch (int)));
  match->insertItem(tr("Crossword Match"), this, SLOT(setCurrentMatch (int)));

#ifdef ZAURUS
#if 0
  zau_use_itimer=0;
  /*
  zau_c3000_mode=1;
  */
#endif
#endif

  zten = new Zten (this);

  // -----------------
  //  menu->insertItem(tr("Dictionary Group"),menuDictionaryGroup);
  menu->insertItem (tr("Default Match"), match);
  incremental_id = menu->insertItem (tr("Incremental Search"), this,
					SLOT(toggleIncremental()));
  autoContent_id = menu->insertItem (tr("Auto Content Display"), this,
					SLOT(toggleAutoContent()));
  clipSearch_id = menu->insertItem (tr("Clipboard Search"), this,
					SLOT(toggleClipSearch()));
  menu->insertItem(tr("Dictionary Setup"),this, SLOT(dicList()));
  menu->insertItem (tr("Font"), this, SLOT(fontList()));
  menu->insertItem (tr("Image scale"), scale);
  menu->insertItem (tr("ToolBar"), menuConfigToolBar);
  menu->insertItem(tr("Search Condition"), this, SLOT (setPreferences (void)));
  menu->insertItem(tr("Dump Current Page"),this, SLOT(dumpCurrent()));
  menu->insertItem(tr("Application Infomation"),this, SLOT(showInfo()));
  menu->insertSeparator();
  menu->insertItem(tr("Exit"),this, SLOT(exitApplication()));
  //----------------
  
  //  menuBar->insertItem (ZPIXMAP(menu_xpm), zten->MenuSearch);
#ifdef ZAURUS
  menuBar->insertItem (tr("&M"), zten->MenuSearch);
  menuBar->insertItem (tr("&C"), zten->MultiSearch);
  menuBar->insertItem (tr("&G"), menuDictionaryGroup);
  //  menuBar->insertItem (ZPIXMAP(gear_xpm), menu);
  menuBar->insertItem (tr("&P"), menu);
#else
  menuBar->insertItem (tr("MenuSearch"), zten->MenuSearch);
  menuBar->insertItem (tr("MultiSearch"), zten->MultiSearch);
  menuBar->insertItem (tr("DictionaryGroup"), menuDictionaryGroup);
  //  menuBar->insertItem (ZPIXMAP(gear_xpm), menu);
  menuBar->insertItem (tr("Preferences"), menu);
#endif

  QToolTip::add(menuBar,"M: Menu Search, C: Multi Search, \nG: Dictionary Group, P: Property");

  config->readConfig (zten);

  setCentralWidget (zten);
  
  dicGroup_id=new int[MAX_DIC_GROUPS+2];  
  setupDictionaryGroupMenu();
  QObject::connect(menuDictionaryGroup, SIGNAL(activated(int)),
		   this, SLOT (setDictionaryGroupId(int)));

  /*
  connect (QApplication::clipboard(), SIGNAL (dataChanged()), this,
	   SLOT(lookup1()));
  */

  connect (zten, SIGNAL (noHistory ()), this, SLOT (disableNavigations()));
  connect (zten, SIGNAL (beginningOfHistory ()), this, SLOT (disableBack ()));
  connect (zten, SIGNAL (endOfHistory ()), this, SLOT (disableForward()));
  connect (zten, SIGNAL (makeHistory ()), this, SLOT (enableBackNavigations()));
  connect (zten, SIGNAL (followHistory ()), this, SLOT (enableNavigations()));

  connect (this, SIGNAL (reloadBooks ()), zten, SLOT (reload ()));

  //  connect (zten, SIGNAL (incrementalIsSet (bool)), this, SLOT (setIncremental(bool)));

  // Receive QCOP message.
#ifdef ZAURUS
  if (config->EnableQCopZtenv) {
    QCopChannel *channel = new QCopChannel( "QPE/Application/ztenv", this );
    connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
	    this, SLOT(dispatchMessage(const QCString&, const QByteArray&)));  
  }
  if (config->EnableQCopZten) {
    QCopChannel *channel = new QCopChannel( "QPE/Application/zten", this );
    connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
	    this, SLOT(dispatchMessage(const QCString&, const QByteArray&)));  
  }
  if (config->EnableQCopZdict) {
    QCopChannel *channel = new QCopChannel( "QPE/Application/zdict", this );
    connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
	    this, SLOT(dispatchMessage(const QCString&, const QByteArray&)));  
  }
#endif

  setSearchTool(config->SearchToolbar);
  setEditTool(config->EditToolbar);
  setBrowseTool(config->BrowseToolbar);
//  setTitleBar(config->EnableTitleBar);

  checkCurrentMatch(config->CurrentMatch);
  checkImageScale(config->ImageScale);

  drawToolBar();
  updateMenuStatus();

#ifndef ZAURUS
  resize( QSize(640, 480).expandedTo(minimumSizeHint()) );
#endif

  //  emit reloadBooks();
  timer = new QTimer();
  connect(timer, SIGNAL(timeout ()), zten, SLOT(diskChanged()));
  timer -> start(0,true);
}

void ZtenWindow::exitApplication(void) {
  fprintf(stderr,"Exit\n");
  app->quit();
}

void ZtenWindow::setupDictionaryGroupMenu() {
  menuDictionaryGroup->clear();
  zten->singleBook.clear(); 
  for(int i=0;i<MAX_DIC_GROUPS+2;i++){
    if (i==DIC_GROUP_SINGLE) {
      //      QPopupMenu *pop=new QPopupMenu(this, config->GroupName[i]);
      QPopupMenu *pop=new QPopupMenu(menuDictionaryGroup);
      dicGroup_id[i]=menuDictionaryGroup->
	insertItem(config->GroupName[i],pop);
      for(int j=0;j<MAX_DIC_GROUPS+2;j++){
	if (j==DIC_GROUP_SINGLE) continue;
	QPopupMenu *pop2=new QPopupMenu(pop);
	connect(pop2, SIGNAL(activated(int)),
		this, SLOT(selectSingleDictionary(int)));
	pop->insertItem(config->GroupName[j],pop2);
	for (QBookEntry* be = zten->books.first (); 
	     be != NULL; be = zten->books.next ()) {
	  if (be->priority[j]>0) {
	    int id=pop2->insertItem(be->getTitle());
	    zten->singleBook[id]=be;
	  }
	}
      }
    } else {
      dicGroup_id[i]=menuDictionaryGroup->
	insertItem(config->GroupName[i]);
    }
  }
  checkDictionaryGroup(QBookEntry::getGroupNumber());
}

void
ZtenWindow::checkDictionaryGroup(int dicGroup)
{
  for(int i=0; i<MAX_DIC_GROUPS+2; i++) {
    int id = menuDictionaryGroup->idAt(i);
    if(id != -1) {
      menuDictionaryGroup->setItemChecked(id, i==dicGroup);
    }
  }
}

void ZtenWindow::setDictionaryGroupId(int id) {
  for(int i=0;i<MAX_DIC_GROUPS+2;i++){
    if (i==DIC_GROUP_SINGLE) continue;
    if (dicGroup_id[i]==id) setDictionaryGroup(i);
  }
}

void ZtenWindow::setDictionaryGroup(int n) {
  QBookEntry::setGroupNumber(n);
  checkDictionaryGroup(n);
  zten->books.sort();
  zten->diskChanged();
  zten->search();
}

void
ZtenWindow::dispatchMessage (const QCString &msg, const QByteArray &data)
{
  if (msg == "lookup(QString)")
    {
      QDataStream stream (data, IO_ReadOnly);
      QString key;
      stream >> key;
      lookup (QString (key));
    }
  else if (msg == "lookup1(QString)")
    {
      QDataStream stream (data, IO_ReadOnly);
      QString key;
      stream >> key;
      lookup1 (QString (key));
    }
  else if (msg == "lookup()" || msg == "honyaku()")
    {
      lookup ();
    }
  else if (msg == "lookup1()")
    {
      lookup1 ();
    }
  else if (msg == "setGroup(QString)")
    {
      QDataStream stream (data, IO_ReadOnly);
      QString key;
      stream >> key;
      setDictionaryGroup (key.toInt());
    }
}

void
ZtenWindow::lookup (QString key)
{
  config->clipSearchMode=false;
  drawToolBar();
  showMaximized();
  raise();
  zten->lookup (key);
}

void
ZtenWindow::lookup ()
{
  config->clipSearchMode=false;
  drawToolBar();
  showMaximized();
  raise();
  zten->lookup (QApplication::clipboard()->text().simplifyWhiteSpace());
}

void
ZtenWindow::lookup1 (QString key) // Don't raise
{
  config->clipSearchMode=true;
  drawToolBar();
  showMaximized();
  zten->lookup (key);
}

void
ZtenWindow::lookup1 () // Don't raise
{
  QString s=QApplication::clipboard()->text().simplifyWhiteSpace();
  if (s.length()==0) return;
  config->clipSearchMode=true;
  drawToolBar();
  showMaximized();
  zten->lookup (s);
}

void ZtenWindow::openTrig() {
#ifdef ZAURUS
  timer2 -> start(0,true);
#endif
}

void ZtenWindow::closeFiles() {
#ifdef ZAURUS
#if 0
  static time_t t0=0;
  time_t t1=time(NULL);
  if (t1-t0>1) {
    t0=t1;
    timer2 -> start(1000,true);
    return;
  }
  t0=t1;

  if (zau_opened) {
    zau_temporary_close_all();
  }
#endif
#endif
}

void
ZtenWindow::resume ()
{
  // Delay for Slow SD?
  //  sleep(1);
  usleep(1000*config->ResumeWait);

  //  setCursor(WaitCursor);
  //  usleep(100000); 
              // Wait until SD ready (0.1 sec for SD, 3 sec for CF ready)
  //  zten->closeBooks();
  //  setCursor(ArrowCursor);
  //  DiskStat::checkDiskReady();

  //  zten->ContentBrowser->setText ("");
  //  zten->EntryList->clear();

  //  zten->reload ();
}

void ZtenWindow::enableNavigations ()
{
  // enable back, forward
  backAction->setEnabled (TRUE);
  forwardAction->setEnabled (TRUE);
}

void
ZtenWindow::enableBackNavigations ()
{
  // enable back, disable forward
  backAction->setEnabled (TRUE);
  forwardAction->setEnabled (FALSE);
}
void
ZtenWindow::disableNavigations ()
{
  // disable back, forward
  backAction->setEnabled (FALSE);
  forwardAction->setEnabled (FALSE);
}
void
ZtenWindow::disableBack ()
{
  // disable back
  backAction->setEnabled (FALSE);
}
void
ZtenWindow::disableForward ()
{
  // disable forward
  forwardAction->setEnabled (FALSE);
}

void ZtenWindow::dicList()
{
  zten->dicList();
}

void ZtenWindow::fontList()
{
  zten->selectFont();
}

void
ZtenWindow::setIncremental (bool on)
{
  checkIncremental(on);
  zten->setIncremental(on);
}

void
ZtenWindow::checkIncremental (bool on)
{
  if (on)
    {
      menu->setItemChecked (incremental_id, true);
    }
  else
    {
      menu->setItemChecked (incremental_id, false);
    }
}

void
ZtenWindow::toggleIncremental ()
{
  if (menu->isItemChecked (incremental_id))
    {
      setIncremental (false);
    }
  else
    {
      setIncremental (true);
    }
}

void
ZtenWindow::setAutoContent (bool on)
{
  checkAutoContent(on);
  zten->setAutoContent(on);
}

void
ZtenWindow::checkAutoContent (bool on)
{
  if (on)
    {
      menu->setItemChecked (autoContent_id, true);
    }
  else
    {
      menu->setItemChecked (autoContent_id, false);
    }
}

void
ZtenWindow::toggleAutoContent ()
{
  if (menu->isItemChecked (autoContent_id))
    {
      setAutoContent (false);
    }
  else
    {
      setAutoContent (true);
    }
}

void
ZtenWindow::setClipSearch (bool on)
{
  checkClipSearch(on);
  zten->setClipSearch(on);
  if (on) {
    timer3->start(500,TRUE);
  } else {
    timer3->stop();
  }
}

void
ZtenWindow::checkClipSearch (bool on)
{
  if (on)
    {
      menu->setItemChecked (clipSearch_id, true);
    }
  else
    {
      menu->setItemChecked (clipSearch_id, false);
    }
}

void
ZtenWindow::toggleClipSearch ()
{
  if (menu->isItemChecked (clipSearch_id))
    {
      setClipSearch (false);
    }
  else
    {
      setClipSearch (true);
    }
}

#if 0
void
ZtenWindow::setTitleBar (bool on)
{
  config->EnableTitleBar=on;
  checkTitleBar(on);
  return;
  if (on) {
    reparent(0,WType_TopLevel,QPoint(0,0));
    fprintf(stderr,"Title On\n");
  } else {
    reparent(0,WType_TopLevel | WStyle_Customize | WStyle_NoBorderEx,
	     QPoint(0,0));
    fprintf(stderr,"Title Off\n");
  }
}

void
ZtenWindow::checkTitleBar (bool on)
{
  if (on)
    {
      menu->setItemChecked (titleBar_id, true);
    }
  else
    {
      menu->setItemChecked (titleBar_id, false);
    }
}

void
ZtenWindow::toggleTitleBar ()
{
  fprintf(stderr,"Toggle Title\n");
  if (menu->isItemChecked (titleBar_id))
    {
      setTitleBar (false);
    }
  else
    {
      setTitleBar (true);
    }
}
#endif

void
ZtenWindow::setEditTool (bool on)
{
  config->EditToolbar=on;
  if (on)
    {
      menuConfigToolBar->setItemChecked (editTool_id, true);
    }
  else
    {
      menuConfigToolBar->setItemChecked (editTool_id, false);
    }
  drawToolBar();
}

void
ZtenWindow::toggleEditTool ()
{
  if (menuConfigToolBar->isItemChecked (editTool_id))
    {
      setEditTool (false);
    }
  else
    {
      setEditTool (true);
    }
}

void
ZtenWindow::setBrowseTool (bool on)
{
  config->BrowseToolbar=on;
  if (on)
    {
      menuConfigToolBar->setItemChecked (browseTool_id, true);
    }
  else
    {
      menuConfigToolBar->setItemChecked (browseTool_id, false);
    }
  drawToolBar();
}

void
ZtenWindow::toggleBrowseTool ()
{
  if (menuConfigToolBar->isItemChecked (browseTool_id))
    {
      setBrowseTool (false);
    }
  else
    {
      setBrowseTool (true);
    }
}

void
ZtenWindow::setSearchTool (bool on)
{
  config->SearchToolbar=on;
  if (on)
    {
      menuConfigToolBar->setItemChecked (searchTool_id, true);
    }
  else
    {
      menuConfigToolBar->setItemChecked (searchTool_id, false);
    }
  drawToolBar();
}

void
ZtenWindow::toggleSearchTool ()
{
  if (menuConfigToolBar->isItemChecked (searchTool_id))
    {
      setSearchTool (false);
    }
  else
    {
      setSearchTool (true);
    }
}

void ZtenWindow::resizeEvent(QResizeEvent*) {
  config->clipSearchMode=false;
  drawToolBar ();
  //  lastClipboard=QApplication::clipboard()->text();
  //  QCopEnvelope e("QPE/trans_tray", "mainexec()");
  fprintf(stderr,"resize\n");
}
/*
void ZtenWindow::showMaximized() {
//  lastClipboard=QApplication::clipboard()->text();
//  QCopEnvelope e("QPE/trans_tray", "mainexec()");
  fprintf(stderr,"showMaximized\n");
}
*/

void
ZtenWindow::drawToolBar ()
{
//  editBox->removeFrom(toolBar);
//  browseBox->removeFrom(toolBar);
  if (config->clipSearchMode && height()*2<app->desktop()->height()) {
    menuBarBase->hide();
    //    menuBar->hide();
    searchBar->hide();
  } else {
    menuBar->show();
    if (menuConfigToolBar->isItemChecked (editTool_id)) {
//      editBox->addTo(toolBar);
      editBar->show();
    } else {
      editBar->hide();
    }
    if (menuConfigToolBar->isItemChecked (browseTool_id)) {
//      browseBox->addTo(toolBar);
      browseBar->show();
    } else {
      browseBar->hide();
    }
    if (menuConfigToolBar->isItemChecked (searchTool_id)) {
      searchBar->show();
    } else {
      searchBar->hide();
    }
    menuBarBase->show();
  };
}

void
ZtenWindow::checkCurrentMatch(int currentMatch)
{
  for(size_t i=0; i<sizeof(match_id)/sizeof(match_id[0]); i++) {
    int id = match->idAt(i);
    if(id != -1) {
      match->setItemChecked(id, match_id[i]==currentMatch);
    }
  }
}

void
ZtenWindow::setCurrentMatch (int id)
{
  int index = match->indexOf (id);
  checkCurrentMatch(match_id[index]);
  config->CurrentMatch=match_id[index];
}

void
ZtenWindow::checkImageScale(int imageScale)
{
  for(size_t i=0; i<sizeof(scales)/sizeof(scales[0]); i++) {
    int id = scale->idAt(i);
    if(id != -1) {
      scale->setItemChecked(id, scales[i]==imageScale);
    }
  }
}

void
ZtenWindow::setImageScale (int id)
{
  int index = scale->indexOf (id);
  checkImageScale(scales[index]);
  zten->setImageScale (scales[index]);
}

void
ZtenWindow::copy ()
{
  zten->copy();
}

void
ZtenWindow::paste ()
{
  zten->paste();
}

void
ZtenWindow::forward ()
{
  zten->forward();
}

void
ZtenWindow::back ()
{
  zten->back();
}

void
ZtenWindow::down ()
{
  zten->down();
}

void ZtenWindow::setPreferences ()
{
  zten->setPreferences();
}

void ZtenWindow::showInfo() {
  zten->showInfo();
}

void ZtenWindow::dumpCurrent() {
  zten->dumpCurrent();
}

void ZtenWindow::updateMenuStatus(void) {
  checkAutoContent(config->AutoContentMode);
  checkClipSearch(config->EnableClipSearch);
  checkIncremental(config->IncrementalMode);
  for(int i=0;i<MAX_DIC_GROUPS;i++){
    menuDictionaryGroup->changeItem(dicGroup_id[i],config->GroupName[i]);
  }
}

void ZtenWindow::selectSingleDictionary(int n) {
  QMap<int, QBookEntry*>::Iterator it=zten->singleBook.find(n);
  if (it==zten->singleBook.end()) return;
  QBookEntry *be=*it;
  for(QBookEntry *be2=zten->books.first(); be2!=NULL; be2=zten->books.next()) {
    if (be2 == be) {
      be2->priority[DIC_GROUP_SINGLE]=1;
    } else {
      be2->priority[DIC_GROUP_SINGLE]=0;
    }
  }
  setDictionaryGroup(DIC_GROUP_SINGLE);
}

void ZtenWindow::checkClipboard() {
  if (!config->EnableClipSearch) {
    timer3->start(500,TRUE);
    return;
  }
  if (clipSearchCount<0) {
    timer3->start(500,TRUE);
    return;
  }

  timer3->stop();

  clipSearchCount++;

  QString t=QApplication::clipboard()->text().simplifyWhiteSpace();;
  if (lastClipboard=="") {
    lastClipboard=t;
    clipSearchCount=0;
  }
  if (t!=lastClipboard) {
    lastClipboard=t;
    clipSearchCount=0;
  } else if (clipSearchCount==2) {
    lookup1();
  } else if (clipSearchCount>10) clipSearchCount=10;

  timer3->start(500,TRUE);
  return;
}
